c-----------------------------------------------------------------------
c --- WIND2D -- Creates a 2D wind field file from a 3D CALMET file
c-----------------------------------------------------------------------
c
c --- WIND2D    Version: 2.02           Level: 091023              MAIN
c
c     Copyright (c) 2006-2009 by Exponent, Inc.
c
c               D. Strimaitis
c
c --- PURPOSE: Reads a sequence of CALMET.DAT or 3D.DAT files and
c              extracts a single 2D layer (1 level or vertical average)
c              and writes field to a direct access file.
c
c --- UPDATES:
c --- V2.01 (090329) to V2.02 (091023)  DGS
c       (1) Perturb one matching latitude in LCC map projection by a
c           small amount when calculating the Cone Constant for the
c           case in which the 2 matching latitudes are equal.
c           Otherwise, the calculation produces a divide-by-zero.
c           Modified:  LCCDWD
c       (2) Processing for multiple 3D.DAT input files was not fully
c           implemented, causing just the first file to be processed
c           Modified:  RD3D
c
c --- V2.0 (081212) to V2.01 (090329)  DGS
c       (1) Fix QA test on location records in header (halted run with
c           sub-grid 3D.DAT data)
c           Modified:  M3D1
c       (2) Add 3D file type to header doc of W2D file
c           Modified:  BLOCKDATA
c
c --- V1.0 (061015) to V2.0 (081212)  DGS
c       (1) Add option to process 3D.DAT files
c           Supports 1-hour timestep and 3D.DAT dataset 2.1 format
c           New:       MET3HD.W2D
c                      M3D1, RD3D, LCCDWD
c           Modified:  CTLDAT.W2D
c                      RDCF, COMP, NEXTFIL, AVG2D
c       (2) Change default output DA file name
c           Modified:  BLOCK DATA
c
c --- WIND2D calls:     SETUP, COMP, FIN
c-----------------------------------------------------------------------
      Program WIND2D
c
      logical lflag

c --- include QA common block
      include 'qa.w2d'

c --- set version and level number of program
      ver='2.02'
      level='091023'

c --- Setup phase -- initialization & program setup operations
      call SETUP

c --- Computational phase -- basic time loop
      call COMP

c --- Termination phase -- program termination functions
      call FIN

      stop
      end

c----------------------------------------------------------------------
      BLOCK DATA
c----------------------------------------------------------------------
c
c --- WIND2D    Version: 2.02     Level: 090329              BLOCKDATA
c ---           D. Strimaitis
c
c----------------------------------------------------------------------
c --- Include parameter statements
      include 'params.w2d'

c --- Include common blocks
      include 'ctldat.w2d'
      include 'windhd.w2d'
      include 'wrkspc.w2d'

c --- /CONTROL/ variables
      data nmetdat/1/, mavg/1/, method/0/, metfm/1/
      data imetdat/1/
      data lcfiles/.FALSE./
      data layerlo/0/, layerup/0/
      data heightlo/0.0/, heightup/0.0/
      data filelst/'WIND2D.LST'/, filew2d/'WIND2D.W2D'/

c --- /WINDHD/ variables
      data dataset/'WIND2D.DAT      '/
      data dataver/'1.0             '/
      data datamod/'2D wind field extracted from CALMET/3D file(s)'/
c
c --- WRKSPC common block
      data nw1/mxnxy/,nw2/mxnxy/

      end


c----------------------------------------------------------------------
c --- Include subroutine packages from CALPUFF
      include 'calpuff.subs'
      include 'calutils.subs'
c----------------------------------------------------------------------


c----------------------------------------------------------------------
      subroutine setup
c----------------------------------------------------------------------
c
c --- WIND2D    Version: 2.02     Level: 061015                  SETUP
c ---           D. Strimaitis
c
c --- PURPOSE:  Coordinate program set-up operations
c
c --- INPUTS:
c
c --- OUTPUT:
c
c --- SETUP called by: (main)
c
c --- SETUP calls:      COMLINE, RDCF, TFER
c----------------------------------------------------------------------
      include 'params.w2d'
      include 'ctldat.w2d'
      include 'qa.w2d'

c --- Local string variables
      character*30 dot30
      character*132 filectl
      character*180 command

      data filectl/'WIND2D.INP'/
      data dot30/'..............................'/

c --- Report progress
      write(*,*)
      write(*,*)'SETUP PHASE'

c --- Get date and time from system
      call DATETM(rdate,rtime,rcpu)

c --- Overwrite default control file name if provided on command line
      call COMLINE(filectl)

c --- Process instructions from the control file
      open(ictl,file=filectl,status='old')
      call RDCF

c --- Delete an existing output DA file
      klen=LEN_TRIM(filew2d)
      command='del "'//filew2d(1:klen)//'"'
      klen=LEN_TRIM(command)
      write(*,*)command(1:klen)//'  if found ...'
      write(*,*)
      call SYSTEM(command(1:klen))

c --- Open list file
      open(ilst,file=filelst)

c --- Report initial information to the list file
c --- Write banner
      write(ilst,5) ver,level
5     format(///,26x,'WIND2D OUTPUT SUMMARY',/,19x,'VERSION:  ',A12,
     1       ' LEVEL:  ',A12///)

c --- Transfer WIND2D control file to list file
      write(ilst,'(a80/)')dot30//' Control File Image '//dot30
      call TFER(ictl,ilst)
      write(ilst,'(a80///)')dot30//' Control File Image '//dot30

      return
      end

c----------------------------------------------------------------------
      subroutine rdcf
c----------------------------------------------------------------------
c
c --- WIND2D    Version: 2.02     Level: 081212                   RDCF
c ---           D. Strimaitis
c
c --- PURPOSE:  Reads control file for WIND2D application
c
c --- UPDATES:
c --- v1.0 (061015) to v2.0 (081212) DGS
c         - Add 3D.DAT option (METFM)
c
c --- INPUTS:
c
c
c --- OUTPUT:
c
c     Common block /CONTROL/
c                   nmetdat,metfm,method,layerlo,layerup,
c                   heightlo,heightup,mavg,
c                   lcfiles,filemet(mxfil),filelst,filew2d
c
c --- RDCF called by: (main)
c --- RDCF calls:     CASE320, RGET320
c----------------------------------------------------------------------
      include 'params.w2d'
      include 'ctldat.w2d'

c --- Local variables
      logical lerror
      character*1 xp,eq
      character*5 case
      character*16 dataset,dataver
      character*320 char320,blnk320,temp320

      data xp/'!'/
      data eq/'='/
      data lerror/.FALSE./

      do i=1,320
         blnk320(i:i)=' '
      enddo

c --- Condition default filenames
      filelst=ADJUSTL(filelst)
      filew2d=ADJUSTL(filew2d)

c --- Process first line of control file
      read(ictl,'(2a16)') dataset,dataver
      if(dataset.NE.'WIND2D.INP      ') then
         write(*,*)'Error in control file for WIND2D application'
         write(*,*)'Expected DATASET name: WIND2D.INP'
         write(*,*)'Found    DATASET name: ',dataset
         stop 'HALTED in RDCF'
      endif
      if(dataver.NE.'1.0             ') then
         write(*,*)'Error in control file for WIND2D application'
         write(*,*)'Expected Version: 2.0'
         write(*,*)'Found    VERSION: ',dataver
         stop 'HALTED in RDCF'
      endif

c --- Set initial values
c --- Local counters
      nmet=0

c --- Read each record and extract information between !delimiters!
20    char320=blnk320
      read(ictl,'(a320)',end=21) char320

c --- Create upper-case version of line
      klast=LEN_TRIM(char320)
      call CASE320('UPPER',char320,klast)

c --- Position of first 2 delimiters
      ixp1=INDEX(char320,xp)
      if(ixp1.EQ.0) then
c ---    No delimiters, skip this line
         goto 20
      else
         i1=ixp1+1
         i2=klast
         ixp2=INDEX(char320(i1:i2),xp)
         if(ixp2.GT.0) ixp2=ixp1+ixp2
      endif

c --- Must have 2 delimiters
      if(ixp1.GT.ixp2) then
c ---    Problem
         write(*,*)'Error in control file for WIND2D application'
         write(*,*)'Single delimiter found on a line:'
         write(*,*)char320(1:klast)
         stop 'HALTED in RDCF'
      endif

c --- Relative position of equals sign within delimiters
      ieq=INDEX(char320(ixp1:ixp2),eq)

c --- Only line with 2 delimiters and no equal sign should be !END!
      if(ieq.EQ.0) then
         if(INDEX(char320,'END').NE.0) then
c ---       Skip this line
            goto 20
         else
c ---       Problem
            write(*,*)'Error in control file for WIND2D application'
            write(*,*)'Delimiter-pair found without an assignment:'
            write(*,*)char320(1:klast)
            stop 'HALTED in RDCF'
         endif
      endif

c --- Set range of assigned value
      ieq=ixp1+ieq-1
      i1=ieq+1
      i2=ixp2-1

c --- Go through list of variables
c --------------------------------

c --- Number of input met files
      if(INDEX(char320,'NMETDAT').NE.0) then
         read(char320(i1:i2),'(i)') nmetdat

c --- List file name
      elseif(INDEX(char320,'FILELST').NE.0) then
         read(char320(i1:i2),'(a)') filelst
         filelst=ADJUSTL(filelst)

c --- Output DA file name
      elseif(INDEX(char320,'FILEW2D').NE.0) then
         read(char320(i1:i2),'(a)') filew2d
         filew2d=ADJUSTL(filew2d)

c --- Met file name
      elseif(INDEX(char320,'FILEMET').NE.0) then
         nmet=nmet+1
         if(nmet.GT.mxfil) then
            write(*,*)'Error in control file for WIND2D application'
            write(*,*)'Too many met files found: ',nmet
            write(*,*)'Increase MXFIL and recompile'
            stop 'HALTED in RDCF'
         endif
         read(char320(i1:i2),'(a)') filemet(nmet)
         filemet(nmet)=ADJUSTL(filemet(nmet))

c --- Case for filename
      elseif(INDEX(char320,'LCFILES').NE.0) then
         temp320=blnk320
         read(char320(i1:i2),'(a)') temp320
         temp320=ADJUSTL(temp320)
         if(temp320(1:1).EQ.'T') then
            lcfiles=.TRUE.
         elseif(temp320(1:1).EQ.'F') then
            lcfiles=.FALSE.
         else
            write(*,*)'Error in control file for WIND2D application'
            write(*,*)'Expected LCFILES = T or F:'
            write(*,*)char320(1:klast)
            stop 'HALTED in RDCF'
         endif

c --- MET MODEL
      elseif(INDEX(char320,'METFM').NE.0) then
         read(char320(i1:i2),'(i)') metfm

c --- METHOD
      elseif(INDEX(char320,'MWIND').NE.0) then
         read(char320(i1:i2),'(i)') method

c --- LAYERLO
      elseif(INDEX(char320,'LAYERLO').NE.0) then
         read(char320(i1:i2),'(i)') layerlo

c --- LAYERUP
      elseif(INDEX(char320,'LAYERUP').NE.0) then
         read(char320(i1:i2),'(i)') layerup

c --- HEIGHTLO
      elseif(INDEX(char320,'HEIGHTLO').NE.0) then
         call RGET320(char320,i1,i2,heightlo)

c --- HEIGHTUP
      elseif(INDEX(char320,'HEIGHTUP').NE.0) then
         call RGET320(char320,i1,i2,heightup)

c --- MAVG
      elseif(INDEX(char320,'MAVG').NE.0) then
         read(char320(i1:i2),'(i)') mavg

c --- Found unexpected variable assignment
      else
         write(*,*)'Error in control file for WIND2D application'
         write(*,*)'Unexpected variable assignment found:'
         write(*,*)char320(1:klast)
         stop 'HALTED in RDCF'
      endif

      goto 20

21    continue

c --- Check number of files
      if(nmet.NE.nmetdat) then
         write(*,*)'Error in control file for WIND2D application'
         write(*,*)'Number of input files found    : ',nmet
         write(*,*)'does not match number expected : ',nmetdat
         stop 'HALTED in RDCF'
      endif

c --- Test information
      if(nmetdat.LE.0) lerror=.TRUE.
      if(metfm.LT.1 .OR. metfm.GT.2) lerror=.TRUE.
      if(method.LT.1 .OR. method.GT.3) lerror=.TRUE.
c      if(mavg.LT.1 .OR. mavg.GT.2) lerror=.TRUE.
      if(mavg.NE.1) lerror=.TRUE.
      if(method.EQ.1) then
         if(layerlo.LE.0) lerror=.TRUE.
         if(layerup.LT.layerlo) lerror=.TRUE.
      endif
      if(method.EQ.2) then
         if(heightlo.LT.0.0) lerror=.TRUE.
         if(heightup.LT.heightlo) lerror=.TRUE.
      endif
      if(method.EQ.3) then
         if(metfm.EQ.2) lerror=.TRUE.
      endif
      if(LERROR) then
         write(*,*)'Error in control file for WIND2D application'
         write(*,*)'Some required information is MISSING or INVALID'
         write(*,*)'nmetdat > 0?         ',nmetdat
         write(*,*)'metfm 1, or 2?       ',metfm
         write(*,*)'mwind 1, 2, or 3?    ',method
         write(*,*)'(mwind=3 not valid with metfm=2)'
c         write(*,*)'mavg 1 or 2?         ',mavg
         write(*,*)'mavg 1?              ',mavg
         if(method.EQ.1) then
            write(*,*)'layerlo > 0?         ',layerlo
            write(*,*)'layerup >= layerlo?  ',layerup
         endif
         if(method.EQ.2) then
            write(*,*)'heightlo > 0.0?      ',heightlo
            write(*,*)'heightup > heightlo? ',heightup
         endif
         stop 'HALTED in RDCF'
      endif

c --- Apply filename case instruction
      if(LCFILES) then
         case='LOWER'
      else
         case='UPPER'
      endif

      temp320=blnk320
      klast=LEN_TRIM(filelst)
      temp320(1:klast)=filelst(1:klast)
      call CASE320(case,temp320,klast)
      filelst(1:klast)=temp320(1:klast)

      temp320=blnk320
      klast=LEN_TRIM(filew2d)
      temp320(1:klast)=filew2d(1:klast)
      call CASE320(case,temp320,klast)
      filew2d(1:klast)=temp320(1:klast)

      do k=1,nmetdat
         temp320=blnk320
         klast=LEN_TRIM(filemet(k))
         temp320(1:klast)=filemet(k)(1:klast)
         call CASE320(case,temp320,klast)
         filemet(k)(1:klast)=temp320(1:klast)
      enddo

      return
      end

c----------------------------------------------------------------------
      subroutine comp
c----------------------------------------------------------------------
c
c --- WIND2D    Version: 2.02     Level: 081212                   COMP
c ---           D. Strimaitis
c
c --- PURPOSE:  Main computational component of program
c
c --- UPDATES:
c --- v1.0 (061015) to v2.0 (081212) DGS
c         - Add 3D.DAT option (METFM)
c
c --- INPUTS:
c             none
c
c --- OUTPUT:
c             none
c
c --- COMP called by: (main)
c
c --- COMP calls:      FILVERS, MET1, MET2, M3D1, LCCDWD,
c                      WRHD1, RDMET, RD3D,
c                      DEDAT, DELTT, AVG2D, WRHD2
c----------------------------------------------------------------------
      include 'params.w2d'
      include 'ctldat.w2d'
      include 'methd.w2d'
      include 'methr.w2d'
      include 'windhd.w2d'
      include 'wrkspc.w2d'

c --- Local Variables
      real xbuf(mxnx,mxny)
      logical lprt, ldbmet

c --- Report progress
      write(*,*)
      write(*,*)'PROCESSING PHASE'
      write(*,*)

c --- Start newest ending date-time at zero (update as data are read)
      ndhrmet=0
      nsecmet=0

c --- Open a scratch file to store comment records from met file
      open(ixxx,status='scratch')

      if(metfm.EQ.1) then
c ---    Process header of first met file
         call FILVERS(iinp,'BIN',filemet(1),'CALMET          ',vercmet)
         open(iinp,file=filemet(1),status='old',form='unformatted')
c ---    Read the header records
         lprt=.false.
         call MET1(lprt)
      elseif(metfm.EQ.2) then
c ---    Process header of first met file
         open(iinp,file=filemet(1),status='old')
c ---    Read the header records
         lprt=.true.
         call M3D1(lprt)
c ---    Compute wind direction rotation from True North to Grid North
         lprt=.false.
         call LCCDWD(lprt)
      endif

c --- Process header information needed for output file
      call MET2

c --- Open the output 2D met file
      open(iw2d,file=filew2d,access='DIRECT',form='UNFORMATTED',
     &     recl=krecl,status='new')

c --- Translate the header information to DA records and write
      call WRHD1

      ldbmet=.false.
c --- Process all data records
10    if(metfm.EQ.1) call RDMET(ndathrm,nsecm,ldbmet,umet,vmet,tmet,
     1           ipgt,htmix,ustar,xmonin,wstar,rmm,tempss,rhoss,
     2           qswss,irhss,ipcode,temp2d,rho2d,qsw2d,irh2d,
     3           ipcode2d,ndhrmete,nsecmete,ndhrmetb,nsecmetb,
     4           wdiv,ieof)
      if(metfm.EQ.2) call RD3D(ndhrmete,nsecmete,ndhrmetb,nsecmetb,
     &                         umet,vmet,zmet3d,ieof)

c --- Done if EOF reached
      if(ieof.EQ.1) goto 20

c --- Show progress
      print 5,ndhrmetb,nsecmetb,ndhrmete,nsecmete
5     format('+','Processing: ',2(i10,i5.4))

c --- DEBUG
      if(ldbmet) then
         idb=nx/2
         jdb=ny/2
         write(ilst,*)
         write(ilst,*)'Processing: ',ndhrmetb,nsecmetb,
     &                               ndhrmete,nsecmete
      endif

c --- Identify record index for this date-time (2 records each time)
      call DEDAT(ndhrmete,nyrmete,njdmete,nhrmete)
      call DELTT(ibyr,ibjd,ibhr,nyrmete,njdmete,nhrmete,nhours)
      hours=FLOAT(nhours)+(nsecmete-ibsec)/3600.
      k=NINT(hours/hrdt)
      if(k.LE.0) then
         write(*,*)'ERROR in COMP: current time period is early'
         write(*,*)'First period starts at   ',ibdathr,ibsec
         write(*,*)'Current period starts at ',ndhrmetb,nsecmetb
         write(*,*)'Current period ends at   ',ndhrmete,nsecmete
         stop
      endif
      krec=(2*k-1)+nheader

c --- Update newest ending date-time
      if(ndhrmete.GT.ndhrmet) then
         ndhrmet=ndhrmete
         nsecmet=nsecmete
      elseif(ndhrmete.EQ.ndhrmet) then
         if(nsecmete.GT.nsecmet) then
            nsecmet=nsecmete
         endif
      endif

c --- Compute winds for layer
      call AVG2D

c --- Write data records
      call WRTR2D(iw2d,krec,uavg,xbuf,mxnx,mxny,nx,ny,'U-LEV   ',
     &            ndhrmetb,nsecmetb,ndhrmete,nsecmete)
      krec=krec+1
      call WRTR2D(iw2d,krec,vavg,xbuf,mxnx,mxny,nx,ny,'V-LEV   ',
     &            ndhrmetb,nsecmetb,ndhrmete,nsecmete)

c --- DEBUG
      if(ldbmet) then
         write(ilst,*)'Output: ',krec,ndhrmetb,nsecmetb,
     &                           ndhrmete,nsecmete
         write(ilst,*)'        ',uavg(idb,jdb),vavg(idb,jdb)
     &                           
      endif

      goto 10

c --- Update header record with the date-time of the actual last period
20    call WRHD2(ndhrmet,nsecmet)

c --- Report selected information to list file
      write(ilst,*)
      write(ilst,*) 'Selected output header information'
      write(ilst,*) '----------------------------------'
      write(ilst,*)
      write(ilst,*) 'Record structure'
      write(ilst,*) 'number of bytes/record    = ',krecl
      write(ilst,*) 'number of header records  = ',nheader
      write(ilst,*) 'number of c132 comment lines = ',ncoml
      write(ilst,*) 'configuration info record = ',iconfig
      write(ilst,*)
      write(ilst,*) 'Met Model Grid'
      write(ilst,*) 'nx,ny,nz,dx = ',nx,ny,nz,xgrid
      write(ilst,*) 'xorig,yorig = ',xorig,yorig
      write(ilst,*) 'pmap,datum  = ',pmap,datum
      write(ilst,*)
      write(ilst,*) 'Data start at'
      write(ilst,*) 'ibdathr,ibsec = ',ibdathr,ibsec,axbtz
      write(ilst,*)
      write(ilst,*) 'Time-step'
      write(ilst,*) 'nsecdt,hrdt = ',nsecdt,hrdt
      write(ilst,*)
      write(ilst,*) 'Number of periods'
      write(ilst,*) 'irlg = ',irlg
      write(ilst,*)

      return
      end

c----------------------------------------------------------------------
      subroutine nextfil(nx,ny,ifound)
c----------------------------------------------------------------------
c
c --- WIND2D    Version: 2.02     Level: 081212                NEXTFIL
c ---           D. Strimaitis
c
c --- PURPOSE:  Prepare the next model data records if
c               multiple files are used
c
c --- UPDATES:
c --- v1.0 (061015) to v2.0 (081212) DGS
c         - Add 3D.DAT option
c
c --- INPUTS:
c              NX,NY - integer - Met grid cell number
c
c     Common block /CONTROL/ variables:
c             FILEMET, NMETDAT, IMETDAT, METFM
c
c --- OUTPUT:
c             IFOUND - integer - Flag indicating if more data records
c                                are available (1=yes)
c
c --- Parameters used:
c        IINP,MXFIL
c
c --- NEXTFIL called by: RDMET, RD3D
c --- NEXTFIL calls:     FILVERS, MET1, M3D1, QAMETHD
c----------------------------------------------------------------------
c --- Include parameters
      include 'params.w2d'

c --- Include common blocks
      include 'ctldat.w2d'
      include 'methd.w2d'

      logical lprt

c --- Close the current file
      close(iinp)

c --- Screen out obvious cases
      if(imetdat.GE.nmetdat) then
         ifound=0
         return
      endif

c --- Open the next data file in the list
      imetdat=imetdat+1
      k=imetdat
      if(metfm.EQ.1) then
         call FILVERS(iinp,'BIN',filemet(k),'CALMET          ',vercmet)
         open(iinp,file=filemet(k),status='old',form='unformatted')
c ---    Read and check the header records
         lprt=.false.
         call MET1(lprt)
      elseif(metfm.EQ.2) then
         open(iinp,file=filemet(k),status='old')
c ---    Read and check the header records
         lprt=.false.
         call M3D1(lprt)
      endif

c --- Make sure that this file is consistent with previous files
      call QAMETHD

c --- Set flag to 'yes'
      ifound=1

      return
      end

c----------------------------------------------------------------------
      subroutine qamethd
c----------------------------------------------------------------------
c
c --- WIND2D    Version: 2.02     Level: 061015                QAMETHD
c ---           D. Strimaitis
c
c --- PURPOSE:  Compares selected variables of output header with
c               those read from header of subsequent met files
c
c --- INPUTS:
c
c      common/METHD/nxm,nym,nzm,xgridm,xorigm,
c         yorigm,rlat0m,rlon0m,xlat1m,xlat2m,
c         zfacem(mxnzp1),
c         irlgmet,
c         ibymet,ibmmet,ibdmet,ibjdmet,ibhmet,ibsmet,
c         ieymet,iemmet,iedmet,iejdmet,iehmet,iesmet,
c         iutmznm,feastm,fnorthm,pmapm,datumm,utmhemm,
c         axbtzm
c
c     common/WINDHD/nx,ny,nz,xgrid,xorig,yorig,
c           zface(mxnzp1),
c           nsecdt,
c           iutmzn,feast,fnorth,pmap,datum,utmhem,
c           clat0,clon0,clat1,clat2,axbtz
c
c --- OUTPUT:   none
c
c --- QAMETHD called by:  COMP
c --- QAMETHD calls:      none
c----------------------------------------------------------------------
      include 'params.w2d'
      include 'methd.w2d'
      include 'windhd.w2d'

      character*16 clat0m,clon0m,clat1m,clat2m

      logical lerror
      lerror=.FALSE.

      if(nx.NE.nxm .OR. ny.NE.nym .OR. nz.NE.nzm .OR.
     &                  xgrid.NE.xgridm) then
         lerror=.TRUE.
         write(*,*)'QA ERROR - Model grid is different'
         write(*,*)'   First grid nx,ny,nz,dx: ',nx,ny,nz,xgrid
         write(*,*)'   Current    nx,ny,nz,dx: ',nxm,nym,nzm,xgridm
      endif
      if(xorig.NE.xorigm .OR. yorig.NE.yorigm) then
         lerror=.TRUE.
         write(*,*)'QA ERROR - Model grid is different'
         write(*,*)'   First grid x0,y0: ',xorig,yorig
         write(*,*)'   Current    x0,y0: ',xorigm,yorigm
      endif


      do i=1,nz+1
         zface(i)=zfacem(i)
      enddo


c --- Compute the number of seconds in 1 timestep (nsecdtm)
      call DELTT(ibymet,ibjdmet,ibhmet,ieymet,iejdmet,iehmet,nhours)
      hrdtmet=FLOAT(nhours)/FLOAT(irlgmet)
      nsecdtm=NINT(3600.*hrdtmet)+(iesmet-ibsmet)/irlgmet
      if(nsecdt.NE.nsecdtm) then
         lerror=.TRUE.
         write(*,*)'QA ERROR - time step is different'
         write(*,*)'   First grid nsecdt: ',nsecdt
         write(*,*)'   Current    nsecdt: ',nsecdtm
      endif

      if(axbtz.NE.axbtzm) then
         lerror=.TRUE.
         write(*,*)'QA ERROR - time zone is different'
         write(*,*)'   First grid axbtz: ',axbtz
         write(*,*)'   Current    axbtz: ',axbtzm
      endif

      if(pmap.NE.pmapm .OR. datum.NE.datumm) then
         lerror=.TRUE.
         write(*,*)'QA ERROR - Coordinate projection is different'
         write(*,*)'   First grid pmap,datum: ',pmap,datum
         write(*,*)'   Current    pmap,datum: ',pmapm,datumm
      endif

      if(pmap.EQ.'UTM     ') then
         if(iutmzn.NE.iutmznm .OR. utmhem.NE.utmhemm) then
            lerror=.TRUE.
            write(*,*)'QA ERROR - Coordinate projection is different'
            write(*,*)'   First grid iutmzn,utmhem: ',iutmzn,utmhem
            write(*,*)'   Current    iutmzn,utmhem: ',iutmznm,utmhemm
         endif
      else

c ---    False offset
         if(feast.NE.feastm .OR. fnorth.NE.fnorthm) then
            lerror=.TRUE.
            write(*,*)'QA ERROR - Coordinate offset is different'
            write(*,*)'   First grid feast,fnorth: ',feast,fnorth
            write(*,*)'   Current    feast,fnorth: ',feastm,fnorthm
         endif

c ---    Initialize the lat-lon character variables to blank
         clat0m='                '
         clon0m=clat0m
         clat1m=clat0m
         clat2m=clat0m

c ---    Projection origin
         if(rlon0m.GE.0.0) then
c ---       West longitude
            write(clon0m(1:8),'(f8.4)') rlon0m
            write(clon0m(9:9),'(a1)') 'W'
         else
c ---       East longitude
            rdum=-rlon0m
            write(clon0m(1:8),'(f8.4)') rdum
            write(clon0m(9:9),'(a1)') 'E'
         endif
         clon0m=ADJUSTL(clon0m)
         if(rlat0m.GE.0.0) then
c ---       North latitude
            write(clat0m(1:8),'(f8.4)') rlat0m
            write(clat0m(9:9),'(a1)') 'N'
         else
c ---       South latitude
            rdum=-rlat0m
            write(clat0m(1:8),'(f8.4)') rdum
            write(clat0m(9:9),'(a1)') 'S'
         endif
         clat0m=ADJUSTL(clat0m)
         if(clat0.NE.clat0m .OR. clon0.NE.clon0m) then
            lerror=.TRUE.
            write(*,*)'QA ERROR - Coordinate origin is different'
            write(*,*)'   First grid clat0,clon0: ',clat0,clon0
            write(*,*)'   Current    clat0,clon0: ',clat0m,clon0m
         endif

c ---    Matching latitudes
         if(xlat1m.GE.0.0) then
c ---       North latitude
            write(clat1m(1:8),'(f8.4)') xlat1m
            write(clat1m(9:9),'(a1)') 'N'
         else
c ---       South latitude
            rdum=-xlat1m
            write(clat1m(1:8),'(f8.4)') rdum
            write(clat1m(9:9),'(a1)') 'S'
         endif
         clat1m=ADJUSTL(clat1m)
         if(xlat2m.GE.0.0) then
c ---       North latitude
            write(clat2m(1:8),'(f8.4)') xlat2m
            write(clat2m(9:9),'(a1)') 'N'
         else
c ---       South latitude
            rdum=-xlat2m
            write(clat2m(1:8),'(f8.4)') rdum
            write(clat2m(9:9),'(a1)') 'S'
         endif
         clat2m=ADJUSTL(clat2m)
         if(clat1.NE.clat1m .OR. clat2.NE.clat2m) then
            write(*,*)'QA WARNING - Matching latitudes are different'
            write(*,*)'   First grid clat1,clat2: ',clat1,clat2
            write(*,*)'   Current    clat1,clat2: ',clat1m,clat2m
         endif

      endif

      if(lerror) then
         stop 'Input met files are inconsistent'
      endif

      return
      end

c----------------------------------------------------------------------
      subroutine m3d1(lecho)
c----------------------------------------------------------------------
c
c --- WIND2D    Version: 2.02           Level: 090329             M3D1
c
c --- PURPOSE: Read the header records from a file in 3D.DAT format 
c
c --- UPDATES:
c --- v2.0 (081212) to v2.01 (090329)   DGS
c     - Revise QA check on (i,j) index of last location record to fix
c       error that halted run with subgrid extraction
c     - CTEMP3D string length reset from 128 to 132
c
c --- INPUTS:
c
c         LECHO - logical       - Control variable for output of header
c                                 record information
c
c        Parameters:
c            ilst, iinp, ixxx, MXNX, MXNY, MXNZ
c
c
c --- OUTPUT:
c
c        Common block /MET3D/ variables:
c            IBYRM,IBJULM,IBHRM,IEYRM,IEJULM,IEHRM,NHRSM3D,
c            NXMET,NYMET,NZMET,DXMET,X1MET,Y1MET,PTOPMET,
c            I13D,J13D,NXP,NYP,NZP,SIGMA(mxnz),
c            XNLAT3D(mxnx,mxny),XELON3D(mxnx,mxny),CONEC3D,
c            DWD3D(mxnx,mxny),ITE3D(mxnx,mxny),
c            feast3D,fnorth3D,iutmzn3D,
c            rnlat03D,relon03D,xlat13D,xlat23D,
c            IOUTM3D,IMET3D,ISTEPPG,
c            NCOMM3D,CNAME3D,CVER3D,CTITLE3D,COMM3D,
c            MAP3D,utmhem3D
c
c        Common block /METHD/ variables:
c      ncom,levmet,dversm,nxm,nym,nzm,xgridm,xorigm,
c      yorigm,xbtzm,xlat0m,xlon0m,llconfm,rlat0m,rlon0m,xlat1m,xlat2m,
c      nssta,nusta,npsta,nlu,iwat1,iwat2,zfacem(mxnzp1),
c      xssta(mxss),yssta(mxss),xlatss(mxss),xlonss(mxss),
c      xusta(mxus),yusta(mxus),xpsta(mxps),ypsta(mxps),
c      z0(mxnx,mxny),ilandu(mxnx,mxny),elev(mxnx,mxny),
c      xlai(mxnx,mxny),nears(mxnx,mxny),lcalgrd,
c      nss,i2dmet,itimes,itimeprf,irlgmet,
c      ibymet,ibmmet,ibdmet,ibjdmet,ibhmet,ibsmet,
c      ieymet,iemmet,iedmet,iejdmet,iehmet,iesmet,
c      iutmznm,feastm,fnorthm,pmapm,datumm,datenm,utmhemm,
c      axbtzm,vercmet

c --- M3D1 called by:  COMP, NEXTFIL
c --- M3D1 calls:      JULDAY, INCR, YR4
c----------------------------------------------------------------------
c --- Include parameters
      include 'params.w2d'

c --- Common blocks
      include 'met3hd.w2d'
      include 'methd.w2d'

      character*132 ctemp3d
C --- BUFF1, BUFF2 - Hold first two records
      character*132 buff1,buff2,blnk132
      character*16 temp16
      character*12 clev
      logical lecho

      do n=1,132
         blnk132(n:n)=' '
      enddo

c --- Prepare for SCRATCH file image
      REWIND(ixxx)

c --- Set map projection variables that are not in 3D.DAT file
      utmhem3d='N'
      iutmzn3D=0
      datum3d='NWS-84'
      daten3d='02-21-2003  '
      feast3D=0.0
      fnorth3D=0.0

c --- Establish dataset version
      buff1=blnk132
      buff2=blnk132
      read(iinp,'(a)')buff1
      read(iinp,'(a)')buff2
      cname3d='                '
      imet3d=-999
      itimem=0
      read(ADJUSTL(buff2(1:12)),'(a12)')cname3d(1:12)
      if(cname3d.EQ.'3D.DAT' .OR. cname3d.EQ.'MM53D.DAT') then
         imet3d=1
      else
         ncomm=0
         if(ADJUSTL(buff1(1:6)).EQ.'3D.DAT')then
            read(buff2,*,err=999)ncomm
            if(ncomm.GE.1) imet3d=2
         else
c ---       Structure does not fit 3D.DAT conventions -- assume
c ---       file is in MM5.DAT format
            imet3d=0
         endif
      endif
      if(imet3d.LT.0) then
         write(*,*)'ERROR in M3D1:  Invalid 3D.DAT file Header'
         write(ilst,*)'ERROR in M3D1:  Invalid 3D.DAT file Header'
         stop
      elseif(imet3d.EQ.0) then
c ---    HALT if old format file.  Grid x,y not in header
c ---    This may be relaxed in the future if the lat/lon coordinates
c ---    are processed to obtain grid x,y.
         write(*,*)'ERROR in M3D1:  Unsupported 3D.DAT file version'
         write(ilst,*)'ERROR in M3D1:  Unsupported 3D.DAT file version'
         stop
      endif

      if(imet3d.EQ.0) then

c ------------------------------------------------------------------
c ---    Old format file
c ------------------------------------------------------------------

c        ---------
c ---    Record #1
c        ---------
         comm3d=blnk132
         read(ADJUSTL(buff1),'(a80,2a12)')
     &            comm3d(1:80),cver3d(1:12),clev 
         ncomm3d=1
         write(ixxx,'(a132)') comm3d
c        ---------
c ---    Record #2
c        ---------
         read(ADJUSTL(buff2),'(5i3)')ioutw,ioutq,ioutc,iouti,ioutg
c        ---------
c ---    Record #3. Projection parameters
c        ---------
         read(iinp,1060)rnlat03d,relon03d,xlat13d,xlat23d  
 1060    format(9x,f6.2,7x,f8.2,8x,f6.1,8x,f6.1)
c        ---------
c ---    Record #4. MPHYSICS
c        ---------
         read(iinp,'(30i3)')inhyd,imphys,icupa,ibltyp,ifrad,isoil,
     &                      ifddaan,ifddaob
c        ---------
c ---    Record #5
c        ---------
         read(iinp,'(4i2,i5,3i4)')ibyrm,ibmom,ibdym,ibhrm,nhrsm3d,
     &                            nxp,nyp,nzp
c        ---------
c ---    Record #6
c        ---------
         read(iinp,'(4i4,4(f8.2))')nx1,ny1,nx2,ny2,
     &                             rxmin,rxmax,rymin,rymax
         nz1=1
         nz2=nzp
c        ---------
c ---    Next NZP records -- MM5 half-sigma levels
c        ---------
         do n=1,nzp
            nn=MIN(n,mxnz)
            read(iinp,'(f6.3)')sigma(nn)
         enddo
c        ---------
c ---    Next NXP * NYP records -- lat., long., elevation, LU
c        ---------
         do j=1,nyp
            jj=MIN(j,mxny)
         do i=1,nxp
            ii=MIN(i,mxnx)
            read(iinp,1065)iindex,jindex,xnlat3d(ii,jj),xelon3d(ii,jj),
     &                     ite3d(ii,jj),ilu3d(ii,jj),flatcrs,flongcrs
 1065       format(2i3,f7.3,f8.3,i5,i3,1x,f7.3,f8.3)
         enddo
         enddo

      else

c ------------------------------------------------------------------
c ---    3D.DAT style files
c ------------------------------------------------------------------

         if(imet3d.EQ.1) then
c           ---------
c ---       Record #1 (title)
c           ---------
            comm3d=blnk132
            read(ADJUSTL(buff1),'(a80)') comm3d(1:80)
            ncomm3d=1
            write(ixxx,'(a132)') comm3d
c           ---------
c ---       Record #2 (Data set name, version and level)
c           ---------
            read(ADJUSTL(buff2),'(3a12)')cname3d,cver3d,ctitle3d

         elseif(imet3d.EQ.2) then
c           ---------
c ---       Record #1 (Dataset name, version and title)
c           ---------
            read(ADJUSTL(buff1),'(2a16,a64)')cname3d,cver3d,ctitle3d
c ---       Set timestamp format
c ---       Version 2 series is end-time in hours, so check for later
c ---       formats
            temp16=ADJUSTL(cver3d)
            if(temp16(1:1).NE.'2') itimem=1

c           ---------
c ---       Record #2 (Number of comment lines)
c           ---------
            read(buff2,*)ncomm3d
c ---       Next "NCOMM3d" lines (comment lines)
            if(ncomm3d.gt.0)then
c ---          Loop over comment records
               comm3d=blnk132
               do i=1,ncomm3d
                  ctemp3d=blnk132
                  read(iinp,'(a132)') ctemp3d
                  write(ixxx,'(a132)') ctemp3d
c ---             Save first line of text for later printing
                  if(i.EQ.1)comm3d=ctemp3d
               enddo
            endif
         endif

c        ---------
c ---    Next header record (MM5 output options)
c        ---------
         read(iinp,'(5i3)')ioutw,ioutq,ioutc,iouti,ioutg 

c        ---------
c ---    Next header record (map projection)
c        ---------
         read(iinp,45) map3d,rnlat03d,relon03d,xlat13d,xlat23d,
     &                 x1met,y1met,dxmet,nxmet,nymet,nzmet
45       format(a4,f9.4,f10.4,2f7.2,2f10.3,f8.3,2i4,i3)

c ---    Polar stereographic projection is called PST in 3D.DAT,
c ---    and is called PS in all coordinate codes.
         if(map3d.EQ.'PST     ') map3d='PS      '

c        ---------
c ---    Next header record (MM5 output options)
c        ---------
c        (Add 13 output options for surface variables)
         read(iinp,'(30i3)') inhyd,imphys,icupa,ibltyp,ifrad,isoil
     &         ,ifddaan,ifddaob
     &         ,igrdt,ipbl,ishf,ilhf,iustr,iswdn
     &         ,ilwdn,ist1,ist2,ist3,ist4,ist5,ist6

c        ---------
c ---    Next header record (MM5 grid data)
c        ---------
c ---    Versions 2.x: hours only - Version 3.x: with seconds
         ibsecm=0
         iesecm=0
         if(itimem.EQ.0) then
            read(iinp,'(i4,3i2,i5,3i4)')
     &           ibyrm,ibmom,ibdym,ibhrm,nhrsm3d,nxp,nyp,nzp
         elseif(itimem.EQ.1) then
            read(iinp,'(i4,3i2,i4,i5,i5,3i4)')
     &           ibyrm,ibmom,ibdym,ibhrm,ibsecm,nhrsm3d,nscm3d,
     &           nxp,nyp,nzp
         endif 

c        ---------
c ---    Next header record (extraction subdomain data)
c        ---------
         read(iinp,30)nx1,ny1,nx2,ny2,nz1,nz2,
     &                rxmin,rxmax,rymin,rymax
30       format(6i4,2f10.4,2f9.4)

c        ---------
c ---    Next NZP records -- MM5 half-sigma levels
c        ---------
         do n=1,nzp
            nn=MIN(n,mxnz)
            read(iinp,'(f6.3)')sigma(nn)
         enddo

c        ---------
c ---    Next NXP * NYP records -- lat., long., elevation, LU
c        ---------
         do j=1,nyp
            jj=MIN(j,mxny)
         do i=1,nxp
            ii=MIN(i,mxnx)
            read(iinp,99)iindex,jindex,XNLAT3D(ii,jj),XELON3D(ii,jj),
     &                   ITE3D(ii,jj),ILU3D(ii,jj)
 99         format(2i4,f9.4,f10.4,i5,i3)
         enddo
         enddo

      endif
c
c --- QA check that last I,J read match expected values
c
      if(iindex.ne.nx2.or.jindex.ne.ny2)then
        write(ilst,*)'ERROR in subr. M3D1 -- I,J do not match ',
     1      'values read on header record'
        write(ilst,*)'Last location I, J = ',iindex,jindex
        write(ilst,*)'Expected      I, J = ',nx2,ny2
        stop
      endif

      if(itimem.EQ.1) then
c ---    HALT if file format includes seconds.  Logic treats hourly
c ---    updates as the smallest timestep between output blocks.  This
c ---    needs to be redone for smaller steps.
         write(*,*)'ERROR in M3D1:  Unsupported 3D.DAT file version'
         write(ilst,*)'ERROR in M3D1:  Unsupported 3D.DAT file version'
         write(ilst,*)'Dataset Version 3 and later may have update'
         write(ilst,*)'periods shorter than 1 hour, which has not been'
         write(ilst,*)'addressed in this program.'
         stop
      endif

c --- Process selected header information
c ---------------------------------------
c --- MM5 data configuration
      ioutm3d=81+10*ioutw+ioutq+ioutc+iouti+ioutg
c --- Grid
      i13d=nx1
      j13d=ny1
c --- Check that array dimensions are not exceeded
      if(nxp.GT.mxnx .OR. nyp.GT.mxny .OR. nzp.GT.mxnz) then
         write(ilst,*)'ERROR in subr. M3D1 -- Array dimensions ',
     1      'are too small for data being read'
         write(ilst,*)'Grid being read  (NXP, NYP, NZP) = ',
     1       nxp,nyp,nzp
         write(ilst,*)'Array dimensions (MXNX, MXNY, MXNZ) = ',
     1       mxnx,mxny,mxnz
         stop
      endif
c --- Check consistency between nz1,nz2, and nzp
      if(nzp.ne.nz2-nz1+1) then
         write(ilst,*)'Error in M3D1: NZ1,NZ2 and NZP not consistent'
         write(ilst,*)'nz1,nz2,nzp:',nz1,nz2,nzp
         stop
      endif
c --- Date and time
      call YR4(ilst,ibyrm,ierrb)
      if(ierrb.NE.0)then
         write(ilst,*)'Error encountered in Subr. YR4 '
         write(ilst,*)'Execution stopping in Subr. M3D1 ',
     1      '-- IERRB = ',ierrb
         stop
      endif
c --- Calculate Julian day
      call JULDAY(ilst,ibyrm,ibmom,ibdym,ibjulm)
c --- Move the time of the first period from the end-time to
c --- the start-time, assuming 1-hour step
      iback=-1
      call INCR(ilst,ibyrm,ibjulm,ibhrm,iback)
      call GRDAY(ilst,ibyrm,ibjulm,ibmom,ibdym)
c --- Compute ending date/time (comment out if using other format)
      ieyrm=ibyrm
      iejulm=ibjulm
      iehrm=ibhrm
      nsecmm5=nscm3d+nhrsm3d*3600
      call INCRS(ilst,ieyrm,iejulm,iehrm,iesecm,nsecmm5)
      call GRDAY(ilst,ieyrm,iejulm,iemom,iedym)


c --------------------------------------------------------
c --- Print the 3D.DAT header information to the list file
c --- (except for the gridded fields)
c --------------------------------------------------------

      if(lecho)then
         write(ilst,101)cname3d,cver3d,ctitle3d,comm3d
101      format(//1x,'Information read from 3D.DAT file'/
     1   5x,'Dataset Name:    ',a12/
     2   5x,'Dataset Version: ',a12/
     3   5x,'Dataset Title:   ',a64/
     4   5x,'First line of comments: ',
     5   8x,a132)
c
         write(ilst,102) inhyd,imphys,icupa,ibltyp,ifrad,isoil,
     :               ifddaan,ifddaob
102      format(/5x,'MM5 physics options: '/
     1           5x,'    Hydrostatic:           ',i2/
     1           5x,'    Moisture scheme:       ',i2/
     1           5x,'    Convection scheme:     ',i2/
     1           5x,'    Boundary layer scheme: ',i2/
     1           5x,'    Radiation scheme       ',i2/
     1           5x,'    Soil scheme:           ',i2/
     1           5x,'    Analysis FDDA:         ',i2/
     1           5x,'    Observation FDDA:      ',i2)


         write(ilst,1021)igrdt,ipbl,ishf,ilhf,iustr,iswdn
     :         ,ilwdn,ist1,ist2,ist3,ist4,ist5,ist6
1021      format(/5x,'MM5 surface variable options: '/
     1           5x,'    Ground temperature:    ',i2/
     1           5x,'    PBL:                   ',i2/
     1           5x,'    Sensible heat flux:    ',i2/
     1           5x,'    Latent heat flux:      ',i2/
     1           5x,'    Frictional velocity:   ',i2/
     1           5x,'    Downward SW radiation: ',i2/
     1           5x,'    Downward LW radiation: ',i2/
     1           5x,'    Soil temp at layer 1:  ',i2/
     1           5x,'    Soil temp at layer 2:  ',i2/
     1           5x,'    Soil temp at layer 3:  ',i2/
     1           5x,'    Soil temp at layer 4:  ',i2/
     1           5x,'    Soil temp at layer 5:  ',i2/
     1           5x,'    Soil temp at layer 6:  ',i2)

         write(ilst,103)1,ioutw,ioutq,ioutc,iouti,ioutg
103      format(/5x,'MM5 output fields (1 = YES; 0 = NO): '/
     1           5x,' Pressure, height,T,Wind speed and direction: ',i2/
     1           5x,'                           Vertical velocity: ',i2/
     1           5x,'                   RH and vapor mixing ratio: ',i2/
     1           5x,'                Cloud and rain mixing ratios: ',i2/
     1           5x,'                  Ice and Snow mixing ratios: ',i2/
     1           5x,'                                     Graupel: ',i2)
        if(imet3d.eq.3) then
c ---      Include seconds (V3.0 - 080205)
           write(ilst,104)ibyrm,ibmom,ibdym,ibhrm,ibsecm,nhrsm3d,
     1                  nscm3d,nxp,nyp,nzp
        else 
           write(ilst,104)ibyrm,ibmom,ibdym,ibhrm,nhrsm3d,
     1                  nxp,nyp,nzp
        endif

104     format(/5x,'Date/time (YYYYMMDDHH) of MM5 data: '/
     1           5x,'                  Start = ',i4,3i2,' (GMT)'/
     2           5x,'              No. hours = ',i4,/
     3           5x,'Extraction Subdomain in MM5 file: '/
     4           5x,'            No. X cells = ',i4/
     5           5x,'            No. Y cells = ',i4/
     6           5x,'            No. layers  = ',i4 )
c
105      format(/5x,'Date/time (YYYYMMDDHH) of MM5 data: '/
     1           5x,'                  Start = ',i4,3i2,' (GMT)'/
     1           5x,'                    and = ',i4,' seconds'/
     2           5x,'  No. hours and seconds = ',i4,' and ',i4/
     3           5x,'Extraction Subdomain in MM5 file: '/
     4           5x,'            No. X cells = ',i4/
     5           5x,'            No. Y cells = ',i4/
     6           5x,'            No. layers  = ',i4 )
c

         write(ilst,106)nx1,ny1,nz1,nx2,ny2,nz2
106      format(/5x,'            Beginning X = ',i4/
     2           5x,'            Beginning Y = ',i4/
     3           5x,'            Beginning Z = ',i4/
     4           5x,'               Ending X = ',i4/
     5           5x,'               Ending Y = ',i4/
     6           5x,'               Ending Z = ',i4)
c
         write(ilst,107)rymin,rymax,rxmin,rxmax
107      format(/5x,' Latitude range : ',f9.4,'  -  ',f9.4/
     :           5x,' Longitude range: ',f10.4, ' - ' ,f10.4)

         write(ilst,108)
108      format(/5x,'MM5 half-sigma levels'/5x,'Level',5x,'Sigma'/)
         do 110 i=1,nzp
            write(ilst,109)i,sigma(i)
110      continue
109      format(4x,i4,6x,f6.4)
      endif

c ---------------------------------------------------
c --- Assign to 'CALMET' header variables in /METHD/
c ---------------------------------------------------

      read(cver3d,'(f)') dversm
      ncom=ncomm3d

c --- Swap 3D.DAT time variables
      irlgmet=nhrsm3d
      xbtzm=0
      axbtzm='UTC+0000'
      ibymet=ibyrm
      ibmmet=ibmom
      ibdmet=ibdym
      ibjdmet=ibjulm
      ibhmet=ibhrm
      ibsmet=ibsecm
      ieymet=ieyrm
      iemmet=iemom
      iedmet=iedym
      iejdmet=iejulm
      iehmet=iehrm
      iesmet=iesecm

c --- Swap 3D.DAT extraction grid
      nxm=nxp
      nym=nyp
      nzm=nzp
      xgridm=dxmet*1000.
      xorigm=(x1met+(FLOAT(i13d)-1.5)*dxmet)*1000.
      yorigm=(y1met+(FLOAT(j13d)-1.5)*dxmet)*1000.
      feastm=feast3d
      fnorthm=fnorth3d
      iutmznm=iutmzn3d
      rlat0m=rnlat03d
      rlon0m=-relon03d
      xlat1m=xlat13d
      xlat2m=xlat23d
      pmapm=map3d
      datumm=datum3d
      datenm=daten3d
      utmhemm=utmhem3d

c --- Fill gridded arrays (these are not used)
c --- Defaults
      zrough=0.25
      xleaf=3.0
      near=0
      do j=1,nyp
      do i=1,nxp
         z0(i,j)=zrough
         ilandu(i,j)=ilu3d(i,j)
         elev(i,j)=FLOAT(ite3d(i,j))
         nears(i,j)=near
      enddo
      enddo

c --- Place a count of the layers into the zface array
c --- (actual heights are in the data records)
      zfacem(1)=0.0
      do k=1,nzp
         zfacem(k)=FLOAT(k)
      enddo
c --- (The following are not "used")
      vercmet='0.0'
      levmet='000000'
      irtype=1
      i2dmet=0
      itimes=0
      nssta=0
      nss=0
      npsta=0
      nusta=0
      lcalgrd=.true.
      iwat1=0
      iwat2=0
      xlat0m=0.0
      xlon0m=0.0
      llconfm=.true.
      nlu=16

      return

999   write(*,*)'3D.DAT with comment line format'
      write(*,*)'But no comment lines'
      write(*,*)'Check/correct the 3D.DAT'
      stop

      end

c----------------------------------------------------------------------
      subroutine LCCDWD(lecho)
c----------------------------------------------------------------------
c
c --- WIND2D    Version: 2.02           Level: 091023           LCCDWD
c
c --- PURPOSE: Compute the wind direction shift from True North to
c              Grid North for LCC map projection
c
c --- UPDATES:
c --- v2.0 (081212) to v2.02 (091023) DGS
c         - Perturb one matching latitude in LCC map projection by a
c           small amount when calculating the Cone Constant for the
c           case in which the 2 matching latitudes are equal.
c           Otherwise, the calculation produces a divide-by-zero.
c
c --- INPUTS:
c
c         LECHO - logical       - Control variable for output
c
c        Parameters:
c            ilst, MXNX, MXNY
c
c        Common block /MET3D/ variables:
c            NXP,NYP,XNLAT3D(mxnx,mxny),XELON3D(mxnx,mxny),
c            relon03D,xlat13D,xlat23D,
c            MAP3D
c
c --- OUTPUT:
c
c        Common block /MET3D/ variables:
c            CONEC3D,DWD3D(mxnx,mxny)
c
c --- LCCDWD called by:  COMP
c --- LCCDWD calls:      none
c----------------------------------------------------------------------
c --- Include parameters
      include 'params.w2d'

c --- Common blocks
      include 'met3hd.w2d'

      logical lecho
      data d2r/0.0174533/

c --- Initialize with zero direction shift
      conec3d=0.0
      do j=1,nyp
      do i=1,nxp
          dwd3d(i,j) = 0
      enddo
      enddo

c --- Non-zero shift for LCC projection
      if(map3d.EQ.'LCC     ') then

c ---    Cone constant
c ---    Use absolute value of latitudes, then adjust for hemisphere
c ---    later
c         conec3d = LOG(COS(d2r*ABS(xlat13d)) / COS(d2r*ABS(xlat23d)))
c         conec3d = conec3d/LOG(TAN(d2r*(45.-0.5*ABS(xlat13d))) /
c     &                         TAN(d2r*(45.-0.5*ABS(xlat23d))))
         a1=ABS(xlat13d)
         a2=ABS(xlat23d)
         if(a1.EQ.a2) then
            a1=a1+.001
            write(*,*)'Calculating CONE constant with equal latitudes'
            write(*,*)'xlat13d, xlat23d = ',xlat13d, xlat23d
            write(*,*)'xlat13d increased by 0.001 degree: ',a1
            write(*,*)
         endif
         conec3d = LOG(COS(d2r*a1) / COS(d2r*a2))
         conec3d = conec3d/LOG(TAN(d2r*(45.-0.5*a1)) /
     &                         TAN(d2r*(45.-0.5*a2)))

         do j=1,nyp
         do i=1,nxp

c ---       All longitues are standard (positive in Eastern Hem.)
            delon = -relon03d + xelon3d(i,j)
c ---       Code to handle 180 degree longitude straddle
            if (delon .GT.  180.) delon=delon-360.
            if (delon .LT. -180.) delon=delon+360.

c ---       Wind direction adjustment to convert from True North to
c ---       Grid North (to be added to input WD)
            if(xnlat3d(i,j) .LT. 0.) then
c ---          Southern Hemisphere
               dwd3d(i,j)=(conec3d*delon)
            else
c ---          Northern Hemisphere
               dwd3d(i,j)=-(conec3d*delon)
            endif
            
         enddo
         enddo

c ---    Output
         if(lecho) then
            write(ilst,*)
            write(ilst,*)'LCCDWD wind direction shift to Grid North'
            write(ilst,*)'LCC Projection Central Longitude: ',relon03d
            write(ilst,*)
            write(ilst,*)'Cell I,J   Lon(deg)   Delta WD(deg)'
            do j=1,nyp
            do i=1,nxp
               write(ilst,*)i,j,xelon3d(i,j),dwd3d(i,j)
            enddo
            enddo
            write(ilst,*)
         endif

      endif

      return
      end

c----------------------------------------------------------------------
      subroutine rd3d(mdathr,msec,mdathrb,msecb,up,vp,zp,ieof)
c----------------------------------------------------------------------
C
c --- WIND2D    Version: 2.02           Level: 091023             RD3D
c
c --- PURPOSE:  Reads data records from the 3D.DAT meteorological file
c
c --- UPDATES:
c --- v2.0 (081212) to v2.02 (091023) DGS
c         - Implement ability to process multiple files (NEXTFIL)
c
c --- INPUTS:
c
c         LECHO - logical       - Control variable for output of header
c                                 record information
c
c        Common block /MET3D/ variables:
c            I13D,J13D,NXP,NYP,NZP,ITE3D(mxnx,mxny),IOUTM3D,IMET3D,
c            DWD3D(mxnx,mxny)
c
c        Parameters:
c            ilst, iinp, MXNX, MXNY, MXNZ
c
c
c --- OUTPUT:
c
c               MDATHR - integer - End date and hour (YYYYJJJHH)
c                 MSEC - integer - End second (SSSS)
c              MDATHRB - integer - Begin date and hour (YYYYJJJHH)
c                MSECB - integer - Begin second (SSSS)
c   UP(mxnx,mxny,mxnz) - real    - U component of the wind (m/s)
c   VP(mxnx,mxny,mxnz) - real    - V-component of the wind (m/s)
c   ZP(mxnx,mxny,mxnz) - real    - Height above ground (m)
c                 IEOF - integer - End of file flag
c
c --- RD3D called by:  COMP
c --- RD3D calls:      JULDAY, YR4, NEXTFIL
c----------------------------------------------------------------------
c --- Include parameters
      include 'params.w2d'

c --- Common blocks
      include 'met3hd.w2d'

      character*80 buff

c --- Only variables needed for output are saved into arrays
c --- Declare 3D profiles
      real up(mxnx,mxny,mxnz),vp(mxnx,mxny,mxnz)
      real zp(mxnx,mxny,mxnz)

      data deg2rad/0.0174533/

c --- Initialize date-time check
      idathr0=-999

c --- Initialize EOF flag
      ieof=0

c --- Loop over each grid cell in extraction subdomain
1     continue
      do j=1,nyp
        do i=1,nxp
       
c ---     2D data record
c ---     --------------
          if(imet3d.EQ.0) then
            read(iinp,60,end=999) myr,mmo,mday,mhr,ix,jx,
     &           pmsl2d,rmm2d,isnow
60          format(4i2,2i3,f7.1,f5.2,i2)
            msec=0
            qsw2d=0.0
            qlw2d=0.0
            temp2d=0.0
            irh2d=0
          else
c ---       Read extended list of surface variables
c ---       (3D.DAT version 2.0 and above)
            if(itimem.EQ.0) then
c ---          Hourly snapshot with endtime listed
               read(iinp,62,end=999) myr,mmo,mday,mhr,ix,jx,
     &                       pmsl2d,rmm2d,isnow,qsw2d,qlw2d,
     &                       temp2d,spechum,wd10,ws10,sstp2
62             format(i4,3i2,2i3,f7.1,f5.2,i2,3f8.1,f8.2,3f8.1)
               msec=0
            elseif(itimem.EQ.1) then
c ---          Snapshot with begin/endtime listed with seconds
               read(iinp,63,end=999) myrb,mmob,mdayb,mhrb,msecb,
     &                               myr,mmo,mday,mhr,msec,ix,jx,
     &                            pmsl2d,rmm2d,isnow,qsw2d,qlw2d,
     &                            temp2d,spechum,wd10,ws10,sstp2
63             format(i4,3i2,i4,i5,3i2,i4,i4,i3,f7.1,f5.2,i2,
     &             3f8.1,f8.2,3f8.1)
            endif
          endif

c ---     Set date/time
          call YR4(ilst,myr,ierr)
          if(ierr.NE.0) stop 'Halted in RD3D'
          call JULDAY(ilst,myr,mmo,mday,mjul)
          mdathr=myr*100000+mjul*100+mhr
c ---     Create start-time if not in file
          if(itimem.EQ.0) then
             myrb=myr
             mmob=mmo
             mdayb=mday
             mhrb=mhr
             msecb=msec
             mdathrb=mdathr
          else
             call YR4(ilst,myrb,ierr)
             if(ierr.NE.0) stop 'Halted in RD3D'
             call JULDAY(ilst,myrb,mmob,mdayb,mjulb)
             mdathrb=myrb*100000+mjulb*100+mhrb
          endif

c ---     Check for consistency in end-time
          if(idathr0.LT.0) then
             idathr0=mdathr
             isec0=msec
          elseif(idathr0.NE.mdathr .OR. isec0.NE.msec) then
             write(*,*)'Halted in RD3D:  Bad dates'
             write(ilst,*)'Halted in RD3D:  Bad dates'
             write(ilst,*)'First       mdathr,msec = ',idathr0,isec0
             write(ilst,*)'Reading mdathr,msec,i,j = ',mdathr,msec,i,j
             stop
          endif

c ---     3D data records
c ---     ---------------
c ---       Read and process
            do n=1,nzp
              if(ioutm3d.eq.81) then
                read(iinp,81) ipress,iz,tk,iwd,ws
81              format (i4,i6,f6.1,i4,f5.1)
              elseif(ioutm3d.eq.82) then
                read(iinp,82) ipress,iz,tk,iwd,ws,
     :                        irh,q
82              format(i4,i6,f6.1,i4,f5.1,i3,f5.2)
              elseif(ioutm3d.eq.83) then
                if(cver3d.eq.'2.1') then
                   read(iinp,830) ipress,iz,tk,iwd,ws,
     :                           irh,q,qc,qr
                else
                   read(iinp,83) ipress,iz,tk,iwd,ws,
     :                          irh,q,qc,qr
                endif
830             format(i4,i6,f6.1,i4,f5.1,i3,f5.2,2(f6.3))
83              format(i4,i6,f6.1,i4,f5.1,i3,3(f5.2))
              elseif(ioutm3d.eq.84) then
                if(cver3d.eq.'2.1') then
                   read(iinp,840) ipress,iz,tk,iwd,ws,
     :                            irh,q,qc,qr,qi,qs
                else
                   read(iinp,84) ipress,iz,tk,iwd,ws,
     :                           irh,q,qc,qr,qi,qs
                endif
840             format(i4,i6,f6.1,i4,f5.1,i3,f5.2,4(f6.3))
84              format(i4,i6,f6.1,i4,f5.1,i3,5(f5.2))
              elseif(ioutm3d.eq.85) then
                if(cver3d.eq.'2.1') then
                   read(iinp,850) ipress,iz,t,iwd,ws,
     :                            irh,q,qc,qr,qi,qs,qg
                else
                   read(iinp,85) ipress,iz,tk,iwd,ws,
     :                           irh,q,qc,qr,qi,qs,qg
                endif
850             format(i4,i6,f6.1,i4,f5.1,i3,f5.2,5(f6.3))
85              format(i4,i6,f6.1,i4,f5.1,i3,6(f5.2))
              elseif(ioutm3d.eq.91) then
                read(iinp,91) ipress,iz,tk,iwd,ws,w
91              format(i4,i6,f6.1,i4,f5.1,f6.2)
              elseif(ioutm3d.eq.92) then
                read(iinp,92) ipress,iz,tk,iwd,ws,w,
     :                        irh,q
92              format(i4,i6,f6.1,i4,f5.1,f6.2,i3,f5.2)

              elseif(ioutm3d.ge.93) then
c ---           May be compressed record
                read(iinp,'(a)')buff
c ---           Read common parts and compression flag 
                if(cver3d.eq.'2.1') then
                   read(buff,930) ipress,iz,tk,iwd,ws,w,
     :                            irh,q,fcomp
                else
                   read(buff,93) ipress,iz,tk,iwd,ws,w,
     :                           irh,q,fcomp
                endif
93              format(i4,i6,f6.1,i4,f5.1,f6.2,i3,3(f5.2))
930             format(i4,i6,f6.1,i4,f5.1,f6.2,i3,f5.2,2(f6.3))

                if(fcomp.gt.-0.0001) then
c ---             Un-compressed form
                  if(ioutm3d.eq.93) then
                     if(cver3d.eq.'2.1') then
                        read(buff,930)ipress,iz,tk,iwd,ws,w,
     :                                irh,q,qc,qr
                     else
                        read(buff,93)ipress,iz,tk,iwd,ws,w,
     :                               irh,q,qc,qr
                     endif
                  elseif(ioutm3d.eq.94) then
                     if(cver3d.eq.'2.1') then
                        read(buff,940)ipress,iz,tk,iwd,ws,w,
     :                                irh,q,qc,qr,qi,qs
                     else
                        read(buff,94)ipress,iz,tk,iwd,ws,w,
     :                               irh,q,qc,qr,qi,qs
                     endif
940                  format(i4,i6,f6.1,i4,f5.1,f6.2,i3,f5.2,4(f6.3))
94                   format(i4,i6,f6.1,i4,f5.1,f6.2,i3,5(f5.2))
                  elseif(ioutm3d.eq.95) then
                     if(cver3d.eq.'2.1') then
c ---                   new moisture format (6.3 instead of 5.2)(050504)
                        read(buff,950) ipress,iz,tk,iwd,ws,w,
     :                                 irh,q,qc,qr,qi,qs,qg
                     else
                        read(buff,95) ipress,iz,tk,iwd,ws,w,
     :                                irh,q,qc,qr,qi,qs,qg
                     endif
95                   format(i4,i6,f6.1,i4,f5.1,f6.2,i3,6(f5.2))
950                  format(i4,i6,f6.1,i4,f5.1,f6.2,i3,f5.2,5(f6.3))
                  endif
                else
c ---             Compressed form
                  if(ioutm3d.eq.93) then
                    qc=0
                    qr=0
                  elseif(ioutm3d.eq.94) then
                    qc=0
                    qr=0
                    qi=0
                    qs=0
                  elseif(ioutm3d.eq.95) then
                    qc=0
                    qr=0
                    qi=0
                    qs=0
                    qq=0
                  endif
                endif
              endif
         
c ---         Convert wd,ws to u,v components and store in grid arrays
c ---         Store temperature in grid array
c ---         (adjust from True North to Grid North)
              wd=FLOAT(iwd)+dwd3d(i,j)
              wdrad = deg2rad*wd
              up(i,j,n) = -ws * sin(wdrad)
              vp(i,j,n) = -ws * cos(wdrad)
c ---         Adjust heights from msl to ht above local ground
              zp(i,j,n) = FLOAT(iz - ite3d(i,j))

c ---       Next nzp
            enddo

c ---   Next nxp
        enddo
c --- Next nyp
      enddo

      return

c --- Ran out of records in 3D.DAT file
999   if(i.EQ.1 .AND. j.EQ.1) then
c ---    Look for another file
         call NEXTFIL(nxp,nyp,ifound)
         if(ifound.EQ.1) then
c ---       Next file is ready to be read
            ieof=0
         else
c ---       Done
            ieof=1
            write(*,*)
            write(*,*)'(RD3D: Reached EOF in last 3D.DAT file)'
         endif
      else
         write(ilst,*) ' Ran out of 3D.DAT data during run while'
         write(ilst,*) ' reading data for cell i,j = ',i,j
         stop
      endif

      if(ieof.EQ.0) goto 1

      return
      end

c----------------------------------------------------------------------
      subroutine met2
c----------------------------------------------------------------------
c
c --- WIND2D    Version: 2.02     Level: 061015                   MET2
c ---           D. Strimaitis
c
c --- PURPOSE:  Creates variables for output header from those read
c               from header of met file
c
c --- INPUTS:
c
c      common/METHD/ncom,nxm,nym,nzm,xgridm,xorigm,
c         yorigm,rlat0m,rlon0m,xlat1m,xlat2m,
c         nssta,nusta,iwat1,iwat2,zfacem(mxnzp1),
c         xssta(mxss),yssta(mxss),
c         xusta(mxus),yusta(mxus),
c         z0(mxnx,mxny),ilandu(mxnx,mxny),elev(mxnx,mxny),
c         xlai(mxnx,mxny),
c         irlgmet,
c         ibymet,ibmmet,ibdmet,ibjdmet,ibhmet,ibsmet,
c         ieymet,iemmet,iedmet,iejdmet,iehmet,iesmet,
c         iutmznm,feastm,fnorthm,pmapm,datumm,datenm,utmhemm,
c         axbtzm
c
c --- OUTPUT:
c
c     common/WINDHD/krecl,ncoml,nx,ny,nz,xgrid,xorig,yorig,
c           nsstn,nustn,kwat1,kwat2,zface(mxnzp1),
c           xsstn(mxss),ysstn(mxss),xustn(mxus),yustn(mxus),
c           z0a(mxnx,mxny),ilandua(mxnx,mxny),eleva(mxnx,mxny),
c           xlaia(mxnx,mxny),
c           nsecdt,irlg,
c           ibyr,ibmo,ibdy,ibjd,ibhr,ibsec,ibdathr,
c           ieyr,iemo,iedy,iejd,iehr,iesec,iedathr,
c           iutmzn,feast,fnorth,pmap,datum,daten,utmhem,
c           clat0,clon0,clat1,clat2,axbtz
c
c --- MET2 called by:  COMP
c --- MET2 calls:      DELTT
c----------------------------------------------------------------------
      include 'params.w2d'
      include 'methd.w2d'
      include 'windhd.w2d'

      ncoml=ncom
      nx=nxm
      ny=nym
      nz=nzm
      xgrid=xgridm
      xorig=xorigm
      yorig=yorigm
      nsstn=nssta
      nustn=nusta
      kwat1=iwat1
      kwat2=iwat2

c --- Define direct access record length (bytes) large enough for
c --- the longest complete header or data record
      kss=24+(nsstn*4)
      kus=24+(nustn*4)
      krecl=24+(nx*ny*4)
      krecl=MAX(krecl,320)
      krecl=MAX(krecl,kss)
      krecl=MAX(krecl,kus)

      irlg=irlgmet
      ibyr=ibymet
      ibmo=ibmmet
      ibdy=ibdmet
      ibjd=ibjdmet
      ibhr=ibhmet
      ibdathr=ibyr*100000+ibjd*100+ibhr
      ibsec=ibsmet
      ieyr=ieymet
      iemo=iemmet
      iedy=iedmet
      iejd=iejdmet
      iehr=iehmet
      iedathr=ieyr*100000+iejd*100+iehr
      iesec=iesmet
      axbtz=axbtzm

      iutmzn=iutmznm
      feast=feastm
      fnorth=fnorthm
      pmap=pmapm
      datum=datumm
      daten=datenm
      utmhem=utmhemm

      do i=1,nsstn
         xsstn(i)=xssta(i)
         ysstn(i)=yssta(i)
      enddo
      do i=1,nustn
         xustn(i)=xusta(i)
         yustn(i)=yusta(i)
      enddo

      do j=1,ny
         do i=1,nx
            z0a(i,j)=z0(i,j)
            eleva(i,j)=elev(i,j)
            ilandua(i,j)=ilandu(i,j)
            xlaia(i,j)=xlai(i,j)
         enddo
      enddo
      do i=1,nz+1
         zface(i)=zfacem(i)
      enddo

c --- Compute the number of seconds in 1 timestep (nsecdt)
c --- Seconds should not be > 3600
      if(ibsec.GT.3600) stop 'MET2: IBSEC > 3600'
      if(iesec.GT.3600) stop 'MET2: IESEC > 3600'
      call DELTT(ibyr,ibjd,ibhr,ieyr,iejd,iehr,nhours)
      hrdt=FLOAT(nhours)/FLOAT(irlg)
      nsecdt=NINT(3600.*hrdt)+(iesec-ibsec)/irlg
c --- Recast as number of hours in 1 timestep (hrdt)
      hrdt=FLOAT(nsecdt)/3600.

c --- Construct strings for lat/lon
      clat0='0N              '
      clon0='0E              '
      clat1=clat0
      clat2=clat0
      if(rlon0m.GE.0.0) then
c ---    West longitude
         write(clon0(1:8),'(f8.4)') rlon0m
         write(clon0(9:9),'(a1)') 'W'
      else
c ---    East longitude
         rdum=-rlon0m
         write(clon0(1:8),'(f8.4)') rdum
         write(clon0(9:9),'(a1)') 'E'
      endif
      clon0=ADJUSTL(clon0)

      if(rlat0m.GE.0.0) then
c ---    North latitude
         write(clat0(1:8),'(f8.4)') rlat0m
         write(clat0(9:9),'(a1)') 'N'
      else
c ---    South latitude
         rdum=-rlat0m
         write(clat0(1:8),'(f8.4)') rdum
         write(clat0(9:9),'(a1)') 'S'
      endif
      clat0=ADJUSTL(clat0)

      if(xlat1m.GE.0.0) then
c ---    North latitude
         write(clat1(1:8),'(f8.4)') xlat1m
         write(clat1(9:9),'(a1)') 'N'
      else
c ---    South latitude
         rdum=-xlat1m
         write(clat1(1:8),'(f8.4)') rdum
         write(clat1(9:9),'(a1)') 'S'
      endif
      clat1=ADJUSTL(clat1)

      if(xlat2m.GE.0.0) then
c ---    North latitude
         write(clat2(1:8),'(f8.4)') xlat2m
         write(clat2(9:9),'(a1)') 'N'
      else
c ---    South latitude
         rdum=-xlat2m
         write(clat2(1:8),'(f8.4)') rdum
         write(clat2(9:9),'(a1)') 'S'
      endif
      clat2=ADJUSTL(clat2)

      return
      end

c----------------------------------------------------------------------
      subroutine WRHD1
c----------------------------------------------------------------------
c
c --- WIND2D    Version: 2.02     Level: 061015                  WRHD1
c ---           D. Strimaitis
c
c --- PURPOSE:  Write met header records to DA file 
c
c --- INPUTS:
c
c     common/WINDHD/krecl,ncoml,
c         nx,ny,nz,xgrid,xorig,yorig,
c         nsstn,nustn,kwat1,kwat2,zface(mxnzp1),
c         xsstn(mxss),ysstn(mxss),xustn(mxus),yustn(mxus),
c         z0a(mxnx,mxny),ilandua(mxnx,mxny),eleva(mxnx,mxny),
c         xlaia(mxnx,mxny),nsecdt,irlg,
c         ibyr,ibmo,ibdy,ibjd,ibhr,ibsec,
c         ieyr,iemo,iedy,iejd,iehr,iesec,
c         iutmzn,feast,fnorth,pmap,datum,daten,utmhem,
c         clat0,clon0,clat1,clat2,axbtz
c
c --- OUTPUT:
c
c     common/WINDHD/nheader,iconfig
c
c --- WRHD1 called by:  COMP
c --- WRHD1 calls:      WRTR1D, WRTR2D, WRTI2D
c----------------------------------------------------------------------
c --- Set size of array for comment lines that are written to 1 record
      parameter (mxcomm=10000)
c --- Include parameters
      include 'params.w2d'
c --- Include common block /CONTROL/ -- Control file data
      include 'ctldat.w2d'
c --- Include common block /METHD/ -- CALMET header record data
      include 'methd.w2d'
c --- Include common block /WINDHD/ -- output header record data
      include 'windhd.w2d'

c --- Local Variables
      real xbuf(mxnx,mxny)
      integer ibuf(mxnx,mxny)
      character*33 blank33
      character*132 comment(mxcomm),blank

      data blank33/'                                 '/

c --- Set blank (132 characters)
      blank(1:33)=blank33
      blank(34:66)=blank33
      blank(67:99)=blank33
      blank(100:132)=blank33

      idum=0

c --- Position scratch file image of comment lines from met file
      REWIND(ixxx)

c --- Number of comment lines (132 characters) per record
      num=krecl/132
c --- Total number of output records for comments
      ntot=ncoml/num
      if((ntot*num).LT.ncoml) ntot=ntot+1
c --- Number of comment lines in last record
      nlast=ncoml-(ntot-1)*num

c --- Slot for first record (will be written at end of subroutine)
      irec=1

c --- Comment record(s)
      do k=1,ntot-1
         do j=1,num
            comment(j)=blank
            read(ixxx,'(a132)') comment(j)
         enddo
         irec=irec+1
         write(iw2d,rec=irec) (comment(j),j=1,num)
      enddo
      if(nlast.GT.0) then
         do j=1,nlast
            comment(j)=blank
            read(ixxx,'(a132)') comment(j)
         enddo
         irec=irec+1
         write(iw2d,rec=irec) (comment(j),j=1,nlast)
      endif

c --- Model configuration record
      irec=irec+1
      iconfig=irec
      write(iw2d,rec=irec) ibyr,ibmo,ibdy,ibhr,ibsec,
     2   ieyr,iemo,iedy,iehr,iesec,axbtz,irlg,nsecdt,
     3   nx,ny,nz,xgrid,xorig,yorig,
     4   nsstn,nustn,kwat1,kwat2,
     5   pmap,datum,daten,feast,fnorth,utmhem,iutmzn,
     6   clat0,clon0,clat1,clat2

c --- Cell face heights (NZ + 1 words)
      irec=irec+1
      call WRTR1D(iw2d,irec,zfacem,nzmp1,'ZFACE   ',
     &            idum,idum,idum,idum)

c --- x, y coordinates of surface stations
c --- (NSSTN words each record)
      if(nsstn.gt.0)then
         irec=irec+1
         call WRTR1D(iw2d,irec,xsstn,nsstn,'XSSTA   ',
     &               idum,idum,idum,idum)
         irec=irec+1
         call WRTR1D(iw2d,irec,ysstn,nsstn,'YSSTA   ',
     &               idum,idum,idum,idum)
      endif

c --- x, y coordinates of upper air stations
c --- (NUSTN words each record)
      if(nustn.gt.0)then
         irec=irec+1
         call WRTR1D(iw2d,irec,xustn,nustn,'XUSTA   ',
     &               idum,idum,idum,idum)
         irec=irec+1
         call WRTR1D(iw2d,irec,yustn,nustn,'YUSTA   ',
     &               idum,idum,idum,idum)
      endif

c --- Surface roughness lengths (NX * NY words)
      irec=irec+1
      call WRTR2D(iw2d,irec,z0a,xbuf,mxnx,mxny,nx,ny,'Z0      ',
     &            idum,idum,idum,idum)

c --- Land use categories (NX * NY words)
      irec=irec+1
      call WRTI2D(iw2d,irec,ilandua,ibuf,mxnx,mxny,nx,ny,'ILANDU  ',
     &            idum,idum,idum,idum)

c --- Elevations (NX * NY words)
      irec=irec+1
      call WRTR2D(iw2d,irec,eleva,xbuf,mxnx,mxny,nx,ny,'ELEV    ',
     &            idum,idum,idum,idum)

c --- Leaf area index (NX * NY words)
      irec=irec+1
      call WRTR2D(iw2d,irec,xlaia,xbuf,mxnx,mxny,nx,ny,'XLAI    ',
     &            idum,idum,idum,idum)

c --- Number of output records in header
      nheader=irec

c --- First record
      write(iw2d,rec=1) krecl,dataset,dataver,datamod,ncoml,nheader,
     &                  iconfig,nmetdat,method,layerlo,layerup,
     &                  heightlo,heightup,mavg

      return
      end
c----------------------------------------------------------------------
      subroutine WRHD2(ndhrmet,nsecmet)
c----------------------------------------------------------------------
c
c --- WIND2D    Version: 2.02     Level: 061015                  WRHD2
c ---           D. Strimaitis
c
c --- PURPOSE:  Update met header records in DA file 
c
c --- INPUTS:
c       NDHRMET - integer       - Ending date-time in file (YYYYJJJHH)
c       NSECMET - integer       - Ending date-time in file (s)
c
c     common/WINDHD/iconfig,
c         nx,ny,nz,xgrid,xorig,yorig,
c         nsstn,nustn,kwat1,kwat2,
c         xlaia(mxnx,mxny),nsecdt,irlg,hrdt,
c         ibyr,ibmo,ibdy,ibjd,ibhr,ibsec,ibdathr,
c         ieyr,iemo,iedy,iejd,iehr,iesec,ibdathr,
c         iutmzn,feast,fnorth,pmap,datum,daten,utmhem,
c         clat0,clon0,clat1,clat2,axbtz
c
c --- OUTPUT:
c
c     common/WINDHD/irlg,ieyr,iemo,iedy,iejd,iehr,iesec,iedathr
c
c --- WRHD2 called by:  COMP
c --- WRHD2 calls:      DEDAT, GRDAY
c----------------------------------------------------------------------
c --- Include parameters
      include 'params.w2d'
c --- Include common block /WINDHD/ -- output header record data
      include 'windhd.w2d'

c --- Update needed?
      if(iedathr.EQ.ndhrmet .AND. iesec.EQ.nsecmet) return

c --- Update ending date-time
      iedathr=ndhrmet
      iesec=nsecmet
      call DEDAT(iedathr,ieyr,iejd,iehr)
      call GRDAY(ilst,ieyr,iejd,iemo,iedy)

c --- Update run length
      call DELTT(ibyr,ibjd,ibhr,ieyr,iejd,iehr,nhours)
      hours=FLOAT(nhours)+(iesec-ibsec)/3600.
      irlg=NINT(hours/hrdt)

c --- Model configuration record
      write(iw2d,rec=iconfig) ibyr,ibmo,ibdy,ibhr,ibsec,
     2   ieyr,iemo,iedy,iehr,iesec,axbtz,irlg,nsecdt,
     3   nx,ny,nz,xgrid,xorig,yorig,
     4   nsstn,nustn,kwat1,kwat2,
     5   pmap,datum,daten,feast,fnorth,utmhem,iutmzn,
     6   clat0,clon0,clat1,clat2

      return
      end

c----------------------------------------------------------------------
      subroutine AVG2D
c----------------------------------------------------------------------
c
c --- WIND2D    Version: 2.02     Level: 081212                  AVG2D
c ---           D. Strimaitis
c
c --- PURPOSE:  Average wind in the vertical 
c
c --- UPDATES
c --- v1.0 (061015) to v2.0 (081212) DGS
c         - Add 3D.DAT option (METFM)
c
c --- INPUTS:
c
c     common/CONTROL/
c         method,layerlo,layerup,heightlo,heightup,mavg,metfm
c     common/WINDHD/
c         nx,ny,nz,zface(mxnzp1)
c     common/METHR/
c         umet(mxnx,mxny,mxnz),vmet(mxnx,mxny,mxnz),htmix(mxnx,mxny)
c         zmet3d(mxnx,mxny,mxnz)
c
c --- OUTPUT:
c
c     common/METHR/
c         uavg(mxnx,mxny),vavg(mxnx,mxny)
c
c --- AVG2D called by:  COMP
c --- AVG2D calls:      
c----------------------------------------------------------------------
c --- Include parameters
      include 'params.w2d'
c --- Include common block /CONTROL/ -- Control file data
      include 'ctldat.w2d'
c --- Include common block /METHR/ -- met data
      include 'methr.w2d'
c --- Include common block /WINDHD/ -- output header record data
      include 'windhd.w2d'

c --- Local variables
      real u(mxnz),v(mxnz),zface3d(mxnz)

      if(method.EQ.1) then
         zlo=zface(layerlo)
         zup=zface(layerup+1)
      elseif(method.EQ.2) then
         zlo=heightlo
         zup=heightup
      elseif(method.EQ.3) then
         zlo=0.0
      else
         stop 'AVG2D: method unnkown'
      endif

      if(mavg.EQ.1) then
c ---    Vector average
         nzp1=nz+1
         do j=1,ny
            do i=1,nx
               if(metfm.EQ.2) then
c ---             Use local 3D.DAT heights for this cell by placing
c ---             heights into zface array (zfaces halfway between
c ---             half-sigma layer heights)
                  zface(1)=0.0
                  zface(nzp1)=zmet3d(i,j,nz)
                  do k=2,nz
                     zface(k)=0.5*(zmet3d(i,j,k)+zmet3d(i,j,k-1))
                  enddo
                  if(method.EQ.1) then
                     zlo=zface(layerlo)
                     zup=zface(layerup+1)
                  endif
               endif
               do k=1,nz
                  u(k)=umet(i,j,k)
                  v(k)=vmet(i,j,k)
               enddo
               if(method.EQ.3) zup=htmix(i,j)
               call ADVWND(u,v,zface,nzp1,zlo,zup,
     &                     uavg(i,j),vavg(i,j))
            enddo
         enddo
      else
         stop 'AVG2D: Only vector averaging is allowed'
      endif

c --- DEBUG
c      idb=nx/2
c      jdb=ny/2
c      write(ilst,*)
c      write(ilst,*)'AVG2D:  Debug for cell i,j = ',idb,jdb
c      write(ilst,*)'Method, zlo,zup = ',method,zlo,zup
c      write(ilst,*)'uavg,vavg = ',uavg(idb,jdb),vavg(idb,jdb)
c      do k=1,nz
c         write(ilst,*)'umet,vmet = ',umet(idb,jdb,k),vmet(idb,jdb,k)
c      enddo

      return
      end

c----------------------------------------------------------------------
      subroutine wrti1d(iomet,irec,idat,nwords,clabel,ndathrb,nsecb,
     &                  ndathre,nsece)
c----------------------------------------------------------------------
c
c --- WIND2D    Version: 2.02     Level: 061015                 WRTI1D
c ---           J. Scire, D. Strimaitis
c
c --- PURPOSE:  Write "NWORDS" of a one-dimensional integer array to
c               DA file
c
c --- INPUTS:
c         IOMET - integer       - Fortran unit number of output file
c          IREC - integer       - Direct access record number
c  IDAT(nwords) - integer array - Array to output
c        NWORDS - integer       - Number of words to write
c        CLABEL - character*8   - Variable name
c   NDATHR[B,E] - integer       - Date and time of data (YYYYJJJHH)
c     NSEC[B,E] - integer       - Seconds for data (SSSS)
c                 [B: beginning of period]
c                 [E: end of period]
c
c --- OUTPUT:  none
c
c --- WRTI1D called by:  WRHD1
c --- WRTI1D calls:      none
c----------------------------------------------------------------------
      integer idat(nwords)
      character*8 clabel
c
      write(iomet,rec=irec)clabel,ndathrb,nsecb,ndathre,nsece,idat
      return
      end
c----------------------------------------------------------------------
      subroutine wrti2d(iomet,irec,idat,ibuf,mxnx,mxny,nx,ny,clabel,
     &                  ndathrb,nsecb,ndathre,nsece)
c----------------------------------------------------------------------
c
c --- WIND2D    Version: 2.02     Level: 061015                 WRTI2D
c ---           J. Scire, D. Strimaitis
c
c --- PURPOSE:  Write NX * NY words of a 2-D integer array to DA file
c
c --- INPUTS:
c            IOMET - integer     - Fortran unit number of output file
c             IREC - integer     - Direct access record number
c  IDAT(mxnx,mxny) - integer arr - Array to output
c      IBUF(nx,ny) - integer arr - Buffer to hold data for output
c        MXNX,MXNY - integers    - Dimensions of data array
c            NX,NY - integers    - Actual size of grid to output
c           CLABEL - character*8 - Variable name
c      NDATHR[B,E] - integer     - Date and time of data (YYYYJJJHH)
c        NSEC[B,E] - integer     - Seconds for data (SSSS)
c                    [B: beginning of period]
c                    [E: end of period]
c
c --- OUTPUT:  none
c
c --- WRTI2D called by:  WRHD1
c --- WRTI2D calls:      none
c----------------------------------------------------------------------
      integer idat(mxnx,mxny),ibuf(nx,ny)
      character*8 clabel
c
      if(nx.eq.mxnx.and.ny.eq.mxny)then
c
c ---    entire array is being used -- write full grid
         write(iomet,rec=irec)clabel,ndathrb,nsecb,ndathre,nsece,idat
      else
c
c ---    only a portion of grid being used -- transfer to buffer
c ---    and write
         do 10 i=1,nx
         do 10 j=1,ny
         ibuf(i,j)=idat(i,j)
10       continue
c
         write(iomet,rec=irec)clabel,ndathrb,nsecb,ndathre,nsece,ibuf
      endif
c
      return
      end
c----------------------------------------------------------------------
      subroutine wrtr1d(iomet,irec,x,nwords,clabel,ndathrb,nsecb,
     &                  ndathre,nsece)
c----------------------------------------------------------------------
c
c --- WIND2D    Version: 2.02     Level: 061015                 WRTR1D
c ---           J. Scire, D. Strimaitis
c
c --- PURPOSE:  Write "NWORDS" of a one-dimensional real array to DA
c               file
c
c --- INPUTS:
c         IOMET - integer     - Fortran unit number of output file
c          IREC - integer       - Direct access record number
c     X(nwords) - real array  - Array to output
c        NWORDS - integer     - Number of words to write
c        CLABEL - character*8 - Variable name
c   NDATHR[B,E] - integer     - Date and time of data (YYYYJJJHH)
c     NSEC[B,E] - integer     - Seconds for data (SSSS)
c                 [B: beginning of period]
c                 [E: end of period]
c
c --- OUTPUT:  none
c
c --- WRTR1D called by:  WRHD1
c --- WRTR1D calls:      none
c----------------------------------------------------------------------
      real x(nwords)
      character*8 clabel
c
      write(iomet,rec=irec)clabel,ndathrb,nsecb,ndathre,nsece,x
      return
      end
c----------------------------------------------------------------------
      subroutine wrtr2d(iomet,irec,x,xbuf,mxnx,mxny,nx,ny,clabel,
     &                  ndathrb,nsecb,ndathre,nsece)
c----------------------------------------------------------------------
c
c --- WIND2D    Version: 2.02     Level: 061015                 WRTR2D
c ---           J. Scire, D. Strimaitis
c
c --- PURPOSE:  Write NX * NY words of a 2-D real array to DA file
c
c --- INPUTS:
c            IOMET - integer     - Fortran unit number of output file
c             IREC - integer     - Direct access record number
c     X(mxnx,mxny) - real array  - Array to output
c      XBUF(nx,ny) - real array  - Buffer to hold data for output
c        MXNX,MXNY - integers    - Dimensions of data array
c            NX,NY - integers    - Actual size of grid to output
c           CLABEL - character*8 - Variable name
c      NDATHR[B,E] - integer     - Date and time of data (YYYYJJJHH)
c        NSEC[B,E] - integer     - Seconds for data (SSSS)
c                    [B: beginning of period]
c                    [E: end of period]
c
c --- OUTPUT:  none
c
c --- WRTR2D called by:  WRHD1
c --- WRTR2D calls:      none
c----------------------------------------------------------------------
      real x(mxnx,mxny),xbuf(nx,ny)
      character*8 clabel
c
      if(nx.eq.mxnx.and.ny.eq.mxny)then
c
c ---    entire array is being used -- write full grid
         write(iomet,rec=irec)clabel,ndathrb,nsecb,ndathre,nsece,x
      else
c
c ---    only a portion of grid being used -- transfer to buffer
c ---    and write
         do 10 i=1,nx
         do 10 j=1,ny
         xbuf(i,j)=x(i,j)
10       continue
c
         write(iomet,rec=irec)clabel,ndathrb,nsecb,ndathre,nsece,xbuf
      endif
c
      return
      end


c----------------------------------------------------------------------
      subroutine case320(mode,c320,nlim)
c----------------------------------------------------------------------
c
c --- WIND2D    Version: 2.02     Level: 061015                CASE320
c ---           D. Strimaitis
c
c --- PURPOSE:  Convert all letters within a character string to 
c               either upper or lower case
c
c --- INPUTS:
c
c             MODE - character*5       - Output directive for either 
c                                        upper or lower case
c             C320 - character*320     - Input character string
c             NLIM - integer           - Length of string (characters)
c
c --- OUTPUT:
c
c             C320 - character*320     - Output character string with
c                                        uniform case
c
c --- CASE320 called by: (utility)
c --- CASE320 calls:      IACHAR, CHAR
c----------------------------------------------------------------------

      character*320 c320
      character*5 mode

      if(nlim.GT.320) then
         write(*,*)'ERROR in Subr. CASE320 - string is too long'
         write(*,*)'Maximum length  : 320'
         write(*,*)'Requested length: ',nlim
         stop 'Program HALTED'
      endif

      if(mode.EQ.'UPPER' .OR. mode.EQ.'upper') then
         do k=1,nlim
            ia=IACHAR(c320(k:k))
            if(ia.GE.97 .AND. ia.LE.122) then
               ia=ia-32
               c320(k:k)=CHAR(ia)
            endif
         enddo

      elseif(mode.EQ.'LOWER' .OR. mode.EQ.'lower') then
         do k=1,nlim
            ia=IACHAR(c320(k:k))
            if(ia.GE.65 .AND. ia.LE.90) then
               ia=ia+32
               c320(k:k)=CHAR(ia)
            endif
         enddo
      else
         write(*,*)'ERROR in Subr. CASE320 - invalid output mode'
         write(*,*)'Expected UPPER or LOWER'
         write(*,*)'Found: ',mode
         stop 'Program HALTED'
      endif

      return
      end

c----------------------------------------------------------------------
      subroutine noblnk320(c320)
c----------------------------------------------------------------------
c
c --- WIND2D    Version: 2.02     Level: 061015              NOBLNK320
c ---           D. Strimaitis
c
c --- PURPOSE:  Remove all blanks from string (left-justify)
c
c --- INPUTS:
c
c             C320 - character*320     - Input character string
c
c --- OUTPUT:
c
c             C320 - character*320     - Output character string with
c
c --- NOBLNK320 called by: (utility)
c --- NOBLNK320 calls:      ADJUSTL, LEN_TRIM
c----------------------------------------------------------------------

      character*320 c320,d320

c --- Blank out work string
      do i=1,320
         d320(i:i)=' '
      enddo

c --- Remove leading blanks
      c320=ADJUSTL(c320)
      last=LEN_TRIM(c320)

c --- Transfer non-blanks into work string
      n=0
      do i=1,last
         if(c320(i:i).NE.' ') then
            n=n+1
            d320(n:n)=c320(i:i)
         endif
      enddo

c --- Pass work string
      c320=d320

      return
      end

c----------------------------------------------------------------------
      subroutine COLUMN320(c320,nlim,cd,ncol,nc,ic)
c----------------------------------------------------------------------
c
c --- WIND2D    Version: 2.02     Level: 061015              COLUMN320
c ---           D. Strimaitis
c
c --- PURPOSE:  Identifies location of all column delimiters in a
c               string and checks for expected number
c
c --- INPUTS:
c
c             C320 - character*320     - Input character string
c             NLIM - integer           - Length of string (characters)
c               CD - character*1       - Column delimiter character
c             NCOL - integer           - Number of columns expected
c
c --- OUTPUT:
c
c               NC - integer           - Number of delimiters
c       IC(0:NLIM) - integer           - Location of delimiters
c
c --- COLUMN320 called by: (utility)
c --- COLUMN320 calls:      none
c----------------------------------------------------------------------

      character*1 cd
      character*320 c320
      integer ic(0:ncol)

      if(nlim.GT.320) then
         write(*,*)'ERROR in Subr. COLUMN320 - string is too long'
         write(*,*)'Maximum length  : 320'
         write(*,*)'Requested length: ',nlim
         stop 'Program HALTED'
      endif

      nc=0
      ic(0)=0
      do k=1,nlim
         if(c320(k:k).EQ.cd) then
            if(nc.LT.ncol) then
               nc=nc+1
               ic(nc)=k
            endif
         endif
      enddo
c --- Last delimiter in line is usually missing
      if(nc.LT.ncol) then
         nc=nc+1
         ic(nc)=nlim+1
      endif
c --- Check number of columns
      if(nc.LT.ncol) then
         write(*,*)'ERROR in Subr. COLUMN320 - '
         write(*,*)'Too few columns in the line'
         write(*,*)'Expected number of columns: ',ncol
         write(*,*)'                     Found: ',nc
         write(*,'(a)')c320(1:nlim)
         stop 'Program HALTED'
      endif

      return
      end

c----------------------------------------------------------------------
      subroutine IRANGE(n,ic,nc,i1,i2)
c----------------------------------------------------------------------
c
c --- WIND2D    Version: 2.02     Level: 061015                 IRANGE
c ---           D. Strimaitis
c
c --- PURPOSE:  Extracts starting and ending position of field for a
c               particular column in a CSV line
c
c --- INPUTS:
c
c                N - integer           - Column number for field
c         IC(0:NC) - integer           - Location of commas
c               NC - integer           - Number of commas
c
c --- OUTPUT:
c
c               I1 - integer           - Starting position of field
c               I2 - integer           - Ending position of field
c
c --- IRANGE called by: (utility)
c --- IRANGE calls:      none
c----------------------------------------------------------------------

      integer ic(0:nc)

      i1=ic(n-1)+1
      i2=ic(n)-1

      return
      end

c----------------------------------------------------------------------
      subroutine rget320(c320,i1,i2,x)
c----------------------------------------------------------------------
c
c --- WIND2D    Version: 2.02     Level: 061015                RGET320
c ---           D. Strimaitis
c
c --- PURPOSE:  Convert character string to a real.  This sub is needed
c               when there is no decimal point.  The LF95 compiler
c               does not do the internal read conversion without the '.'
c               Also, determine when exponential notation is present.
c
c --- INPUTS:
c
c             C320 - character*320     - Input character string
c               i1 - integer           - First character of substring
c               i2 - integer           - Last character of substring
c
c --- OUTPUT:
c
c                X - real              - Output real
c
c --- RGET320 called by: (utility)
c --- RGET320 calls:      none
c----------------------------------------------------------------------
      character*320 c320
      character*20 c20, blnk20
      data blnk20/'                    '/

      if(i1.GT.i2) return

      c20=blnk20
      c20=c320(i1:i2)

c --- Left justify field
      c20=ADJUSTL(c20)

c --- Test for exponential notation
      ie=MAX(INDEX(c20,'e'),INDEX(c20,'E'))

      if(ie.EQ.0) then
c ---    Simple real number
         ipt=INDEX(c20,'.')
         if(ipt.EQ.0) then
c ---       Decimal point NOT found
            ipt=LEN_TRIM(c20)+1
            c20(ipt:ipt)='.'
         endif
c ---    Decimal point in place: internal read OK
         read(c20,'(f)') x

      else
c ---    Exponential notation
         ipt=INDEX(c20,'.')
         if(ipt.EQ.0) then
c ---       Decimal point NOT found
            do k=20,ie+1,-1
               km1=k-1
               c20(k:k)=c20(km1:km1)
            enddo
            c20(ie:ie)='.'
         endif
c ---    Decimal point in place: internal read OK
         read(c20,'(e)') x

      endif

      return
      end



c----------------------------------------------------------------------
      subroutine mlines320(io,char320)
c----------------------------------------------------------------------
c
c --- WIND2D    Version: 2.02     Level: 061015              MLINES320
c ---           D. Strimaitis
c
c --- PURPOSE:  Reads additional records from control file format
c               file when an input assignment extends to more than
c               one line
c
c --- INPUTS:
c               IO - integer      - File unit number
c          CHAR320 - char*320     - Current content of variable
c                                   assignment string
c
c --- OUTPUT:
c          CHAR320 - char*320     - Updated content of variable
c                                   assignment string
c
c --- MLINES320 called by: (main)
c --- MLINES320 calls:     NOBLNK320, CASE320
c----------------------------------------------------------------------
      character*1 cm,xp
      character*180 char180,blnk180
      character*320 char320,blnk320,work320

      data cm/','/
      data xp/'!'/

      do i=1,320
         blnk320(i:i)=' '
      enddo
      blnk180=blnk320(1:180)

c --- Remove all characters that precede the first '!'
      work320=blnk320
      work320=char320(INDEX(char320,xp):320)
      char320=ADJUSTL(work320)

10    klast=LEN_TRIM(char320)

c --- Check current assignment string to see if it is complete
      if(char320(klast:klast).EQ.xp) then
c ---    Assignment is complete
         return
      elseif(char320(klast:klast).EQ.cm) then
c ---    Append next line
         char180=blnk180
         read(io,'(a180)',end=999) char180
         work320=blnk320
         work320(1:180)=char180
         call NOBLNK320(work320)
         klast2=LEN_TRIM(work320)
         call CASE320('UPPER',work320,klast2)
         i1=klast+1
         i2=klast+klast2
         if(i2.GT.320) then
            write(*,*)'ERROR -- assignment string exceeds 320 ',
     &                'characters.  Total number of characters ',
     &                'reached: ',i2
            write(*,*)char320
            write(*,*)work320
            stop 'HALTED in subroutine MLINES320'
         endif
         char320(i1:i2)=work320(1:klast2)
         goto 10
      else
         write(*,*)'ERROR -- completed assignment string must end ',
     &             'with ! and a continued assignment string must ',
     &             'end with a comma'
         write(*,*)char320
         stop 'HALTED in subroutine MLINES320'
      endif

999   write(*,*)'ERROR -- End-of-File encountered when completing ',
     &          'assignment string'
      write(*,*)char320
      stop 'HALTED in subroutine MLINES320'

      end


c----------------------------------------------------------------------
      subroutine rhs320(char1,char320)
c----------------------------------------------------------------------
c
c --- WIND2D    Version: 2.02     Level: 061015                 RHS320
c ---           D. Strimaitis
c
c --- PURPOSE:  Removes characters from the left-hand side of a
c               string up through "char1", and returns the RHS
c               that remains
c
c --- INPUTS:
c            CHAR1 - char*1       - LHS character limit
c          CHAR320 - char*320     - Current content of variable
c                                   assignment string
c
c --- OUTPUT:
c          CHAR320 - char*320     - Updated content of variable
c                                   assignment string
c
c --- RHS320 called by: (main)
c --- RHS320 calls:     none
c----------------------------------------------------------------------
      character*1 char1
      character*320 char320,work320

      do i=1,320
         work320(i:i)=' '
      enddo

      i1=INDEX(char320,char1)+1
      i2=LEN_TRIM(char320)
      work320=char320(i1:i2)
      char320=ADJUSTL(work320)

      return
      end

c----------------------------------------------------------------------
      subroutine omit320(char1,char320)
c----------------------------------------------------------------------
c
c --- WIND2D    Version: 2.02     Level: 061015                OMIT320
c ---           D. Strimaitis
c
c --- PURPOSE:  Removes all instances of a character from a string
c
c --- INPUTS:
c            CHAR1 - char*1       - Character removed
c          CHAR320 - char*320     - Current content of variable
c                                   assignment string
c
c --- OUTPUT:
c          CHAR320 - char*320     - Updated content of variable
c                                   assignment string
c
c --- OMIT320 called by: (main)
c --- OMIT320 calls:     none
c----------------------------------------------------------------------
      character*1 char1
      character*320 char320,work320

      do i=1,320
         work320(i:i)=' '
      enddo

      kk=0
      klast=LEN_TRIM(char320)
      do k=1,klast
         if(char320(k:k).NE.char1) then
            kk=kk+1
            work320(kk:kk)=char320(k:k)
         endif
      enddo
      char320=work320

      return
      end

c----------------------------------------------------------------------
      subroutine tfer(iinp,iout)
c----------------------------------------------------------------------
c
c --- WIND2D    Version: 2.02     Level: 061015                   TFER
c ---           D. Strimaitis
c
c --- PURPOSE:  Transfers one file image to another file
c
c --- INPUTS:
c             IINP - integer      - Unit number for input file
c             IOUT - integer      - Unit number for output file
c
c --- OUTPUT:
c
c
c --- TFER called by: (main)
c --- TFER calls:     none
c----------------------------------------------------------------------

c --- Local variables
      character*640 char640,blnk640
      do i=1,640
         blnk640(i:i)=' '
      enddo

c --- Back to start of file
      REWIND(iinp)

c --- Read and write each record
20    char640=blnk640
      read(iinp,'(a640)',end=21) char640
      klast=LEN_TRIM(char640)
      write(iout,'(a)') char640(1:klast)
      goto 20

21    close(iinp)

      return
      end

c----------------------------------------------------------------------
      subroutine fin
c----------------------------------------------------------------------
c
c --- WIND2D   Version: 2.02     Level: 061015                     FIN
c ---          D. Strimaitis
c
c --- PURPOSE:  Run termination routine -- compute runtime,
c               write last day processed
c
c --- INPUTS:  none
c
c --- OUTPUT:  none
c
c --- FIN called by:  (main)
c --- FIN calls:      DATETM, GRDAY, JULDAY, DELTT, INDECR, YR4C
c----------------------------------------------------------------------
c --- Include parameters
      include 'params.w2d'
      include 'windhd.w2d'
      include 'qa.w2d'

      character*8 rtime2
      character*10 rdate2

      write(*,*)
      write(*,*)'ENTERING TERMINATION PHASE'

c --- Last day/hour processed
      write(ilst,5)ieyr,iemo,iedy,iejd,iehr,iesec
5     format(//2x,'LAST PERIOD PROCESSED ENDS AT:'/5x,
     1       '  Year: ',i4,2x,'Month: ',i2,3x,'Day: ',i2,3x,
     2       'Julian day: ',i3,3x,'Hour: ',i2,3x,'Second: ',i4)

c --- Get system date & time at end of run
      call DATETM(rdate2,rtime2,rcpu)

c --- Compute runtime
      read(rtime(1:2),10)ihr1
      read(rtime(4:5),10)imin1
      read(rtime(7:8),10)isec1
10    format(i2)
      t1=ihr1*3600.+imin1*60.+isec1

      read(rtime2(1:2),10)ihr2
      read(rtime2(4:5),10)imin2
      read(rtime2(7:8),10)isec2
      t2=ihr2*3600.+imin2*60.+isec2

      if(rdate.eq.rdate2)then
         delt=t2-t1
      else
         read(rdate(1:2),10)imo1
         read(rdate(4:5),10)iday1
         read(rdate(7:10),'(i4)')iyr1
         call julday(ilst,iyr1,imo1,iday1,ijul1)

         read(rdate2(1:2),10)imo2
         read(rdate2(4:5),10)iday2
         read(rdate2(7:10),'(i4)')iyr2
         call julday(ilst,iyr2,imo2,iday2,ijul2)

c ---    Compute no. hours from beg. of first hour of run to
c ---    ending hour of ending day of the run
         call deltt(iyr1,ijul1,ihr1,iyr2,ijul2,ihr2,idelhr)

c ---    Adjust for minutes and seconds
         delt=idelhr*3600.-imin1*60.-isec1+imin2*60.+isec2
      endif

c --- On the PC, the runtime and CPU time are the same
c --- (DATETM provides RCPU = 0.0 on the PC)
      if(rcpu.eq.0.0)rcpu=delt

c --- Report current date
      write(ilst,1402)rtime2,rdate2,delt,rcpu
1402  format(//2x,'End of run -- Clock time: ',a8/
     1         2x,'                    Date: ',a10//
     2         2x,'      Elapsed clock time: ',f10.1,' (seconds)'//
     3         2x,'                CPU time: ',f10.1,' (seconds)')

      return
      end
